# -*- coding: utf-8 -*-
"""
Special module, which provide access to all registered URLs, defined in the
main application.
"""
__all__ = ('set_urlconf', 'get_urlconf')

_urlconfs = {}


def set_urlconf(urlconf_data):
    """
    Set the _urlconf for the current thread.
    """
    _urlconfs['data'] = urlconf_data


def get_urlconf(default=None):
    """
    Return the root data from the _urlconf variable, if it has been
    changed from the default one.
    """
    return _urlconfs.get('data', default)



# -*- coding: utf-8 -*-
"""
Field entities, implemented for support SQLAlchemy ORM.

Every class, represented here, is associated with one certain field type of
table relatively to SQLAlchemy ORM. Each of them field also used later for
serializing/deserializing object of ORM.
"""
from aiorest_ws.conf import settings
from aiorest_ws.db.orm import fields
from aiorest_ws.utils.fields import method_overridden

from sqlalchemy import TypeDecorator

__all__ = (
    'IntegerField', 'BigIntegerField', 'SmallIntegerField', 'BooleanField',
    'NullBooleanField', 'CharField', 'EnumField', 'FloatField', 'DecimalField',
    'TimeField', 'DateField', 'DateTimeField', 'IntervalField', 'JSONField',
    'ModelField', 'PickleField', 'LargeBinaryField', 'ReadOnlyField',
    'ListField', 'DictField', 'HStoreField', 'SerializerMethodField'
)


class IntegerField(fields.IntegerField):
    pass


class BigIntegerField(fields.BigIntegerField):
    pass


class SmallIntegerField(fields.SmallIntegerField):
    pass


class BooleanField(fields.BooleanField):
    pass


class CharField(fields.CharField):
    pass


class EnumField(fields.ChoiceField):
    pass


class FloatField(fields.FloatField):
    pass


class NullBooleanField(fields.NullBooleanField):
    pass


class DecimalField(fields.DecimalField):
    pass


class TimeField(fields.TimeField):
    pass


class DateField(fields.DateField):
    pass


class DateTimeField(fields.DateTimeField):
    pass


class IntervalField(fields.TimeDeltaField):
    pass


class ListField(fields.ListField):
    pass


class DictField(fields.DictField):
    pass


class HStoreField(fields.HStoreField):
    pass


class JSONField(fields.JSONField):
    pass


class ModelField(fields.ModelField):
    """
    A generic field that can be used against an arbitrary model field.
    This is used by `ModelSerializer` when dealing with custom model fields,
    that do not have a serializer field to be mapped to.

    For more details recommended to read the next page in SQLAlchemy docs:
    http://docs.sqlalchemy.org/en/latest/core/type_api.html
    """
    def __init__(self, model_field, **kwargs):
        super(ModelField, self).__init__(model_field, **kwargs)
        self.field_type = self.model_field.type

    @property
    def dialect(self):
        return settings.SQLALCHEMY_ENGINE.dialect

    def type_has_custom_behaviour(self, func_name):
        return hasattr(self.field_type, func_name) and \
            method_overridden(func_name, TypeDecorator, self.field_type)

    def to_internal_value(self, data):
        # Custom implementation of converting data for database
        if self.type_has_custom_behaviour('process_bind_param'):
            return self.field_type.process_bind_param(data, self.dialect)

        processor_func = self.field_type.bind_processor(self.dialect)
        if processor_func:
            return processor_func(data)
        return data

    def to_representation(self, obj):
        # Custom implementation of representation of data
        if self.type_has_custom_behaviour('process_result_value'):
            return self.field_type.process_result_value(obj, self.dialect)

        processor_func = self.field_type.result_processor(
            self.dialect, self.field_type
        )
        if processor_func:
            return processor_func(obj)
        return obj


class ReadOnlyField(fields.ReadOnlyField):
    pass


class PickleField(fields.PickleField):
    pass


class LargeBinaryField(fields.LargeBinaryField):
    pass


class SerializerMethodField(fields.SerializerMethodField):
    pass



# -*- coding: utf-8 -*-
"""
Functions to parse datetime objects.
"""
# We're using regular expressions rather than time.strptime because:
# - They provide both validation and parsing.
# - They're more flexible for datetimes.
# - The date/datetime/time constructors produce friendlier error messages.
import datetime
import re

from aiorest_ws.utils.date.timezone import utc, get_fixed_timezone

__all__ = [
    'date_re', 'time_re', 'datetime_re', 'standard_duration_re',
    'iso8601_duration_re', 'parse_date', 'parse_time', 'parse_datetime',
    'parse_timedelta'
]

date_re = re.compile(
    r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})$'
)

time_re = re.compile(
    r'(?P<hour>\d{1,2}):(?P<minute>\d{1,2})'
    r'(?::(?P<second>\d{1,2})(?:\.(?P<microsecond>\d{1,6})\d{0,6})?)?'
)

datetime_re = re.compile(
    r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})'
    r'[T ](?P<hour>\d{1,2}):(?P<minute>\d{1,2})'
    r'(?::(?P<second>\d{1,2})(?:\.(?P<microsecond>\d{1,6})\d{0,6})?)?'
    r'(?P<tzinfo>Z|[+-]\d{2}(?::?\d{2})?)?$'
)

standard_duration_re = re.compile(
    r'^'
    r'(?:(?P<days>-?\d+) (days?, )?)?'
    r'((?:(?P<hours>\d+):)(?=\d+:\d+))?'
    r'(?:(?P<minutes>\d+):)?'
    r'(?P<seconds>\d+)'
    r'(?:\.(?P<microseconds>\d{1,6})\d{0,6})?'
    r'$'
)

# Support the sections of ISO 8601 date representation that are accepted by
# timedelta
iso8601_duration_re = re.compile(
    r'^(?P<sign>[-+]?)'
    r'P'
    r'(?:(?P<days>\d+(.\d+)?)D)?'
    r'(?:T'
    r'(?:(?P<hours>\d+(.\d+)?)H)?'
    r'(?:(?P<minutes>\d+(.\d+)?)M)?'
    r'(?:(?P<seconds>\d+(.\d+)?)S)?'
    r')?'
    r'$'
)


def parse_date(value):
    """
    Parses a string and return a datetime.date.

    Raises ValueError if the input is well formatted but not a valid date.
    Returns None if the input isn't well formatted.
    """
    match = date_re.match(value)
    if match:
        kw = {k: int(v) for k, v in iter(match.groupdict().items())}
        return datetime.date(**kw)


def parse_time(value):
    """
    Parses a string and return a datetime.time.

    This function doesn't support time zone offsets.

    Raises ValueError if the input is well formatted but not a valid time.
    Returns None if the input isn't well formatted, in particular if it
    contains an offset.
    """
    match = time_re.match(value)
    if match:
        kw = match.groupdict()
        if kw['microsecond']:
            kw['microsecond'] = kw['microsecond'].ljust(6, '0')
        kw = {k: int(v) for k, v in iter(kw.items()) if v is not None}
        return datetime.time(**kw)


def parse_datetime(value):
    """
    Parses a string and return a datetime.datetime.

    This function supports time zone offsets. When the input contains one,
    the output uses a timezone with a fixed offset from UTC.

    Raises ValueError if the input is well formatted but not a valid datetime.
    Returns None if the input isn't well formatted.
    """
    match = datetime_re.match(value)
    if match:
        kw = match.groupdict()
        if kw['microsecond']:
            kw['microsecond'] = kw['microsecond'].ljust(6, '0')
        tzinfo = kw.pop('tzinfo')
        if tzinfo == 'Z':
            tzinfo = utc
        elif tzinfo is not None:
            offset_mins = int(tzinfo[-2:]) if len(tzinfo) > 3 else 0
            offset = 60 * int(tzinfo[1:3]) + offset_mins
            if tzinfo[0] == '-':
                offset = -offset
            tzinfo = get_fixed_timezone(offset)
        kw = {k: int(v) for k, v in iter(kw.items()) if v is not None}
        kw['tzinfo'] = tzinfo
        return datetime.datetime(**kw)


def parse_timedelta(value):
    """
    Parse a string into a timedelta object.
    """
    string = value.strip()

    if string == "":
        raise TypeError("'%s' is not a valid time interval" % string)
    # This is the format we get from sometimes Postgres, sqlite,
    # and from serialization
    d = re.match(
        r'^((?P<days>[-+]?\d+) days?,? )?(?P<sign>[-+]?)(?P<hours>\d+):'
        r'(?P<minutes>\d+)(:(?P<seconds>\d+(\.\d+)?))?$',
        str(string)
    )
    if d:
        d = d.groupdict(0)
        if d['sign'] == '-':
            for k in 'hours', 'minutes', 'seconds':
                d[k] = '-' + d[k]
        d.pop('sign', None)
    else:
        # This is the more flexible format
        d = re.match(
            r'^((?P<weeks>-?((\d*\.\d+)|\d+))\W*w((ee)?(k(s)?)?)(,)?\W*)?'
            r'((?P<days>-?((\d*\.\d+)|\d+))\W*d(ay(s)?)?(,)?\W*)?'
            r'((?P<hours>-?((\d*\.\d+)|\d+))\W*h(ou)?(r(s)?)?(,)?\W*)?'
            r'((?P<minutes>-?((\d*\.\d+)|\d+))\W*m(in(ute)?(s)?)?(,)?\W*)?'
            r'((?P<seconds>-?((\d*\.\d+)|\d+))\W*s(ec(ond)?(s)?)?)?\W*$',
            str(string)
        )
        if not d:
            raise TypeError("'%s' is not a valid time interval" % string)
        d = d.groupdict(0)

    return datetime.timedelta(**dict(((k, float(v)) for k, v in d.items())))


def parse_duration(value):
    """
    Parses a duration string and returns a datetime.timedelta.
    The preferred format for durations in Django is '%d %H:%M:%S.%f'.
    Also supports ISO 8601 representation.
    """
    match = standard_duration_re.match(value)
    if not match:
        match = iso8601_duration_re.match(value)
    if match:
        kw = match.groupdict()
        sign = -1 if kw.pop('sign', '+') == '-' else 1
        if kw.get('microseconds'):
            kw['microseconds'] = kw['microseconds'].ljust(6, '0')
        kw = {k: float(v) for k, v in kw.items() if v is not None}
        return sign * datetime.timedelta(**kw)



# -*- coding: utf-8 -*-
"""
Helper functions for mapping model fields to a dictionary or any other
types, which help to serialize or deserialize data.
"""
from aiorest_ws.db.orm.sqlalchemy.validators import UniqueORMValidator, \
    ORMFieldValidator

from sqlalchemy import types, Column
from aiorest_ws.utils.field_mapping import needs_label
from aiorest_ws.utils.text import capfirst


__all__ = (
    'STRING_TYPES', 'get_detail_view_name', 'get_field_kwargs',
    'get_relation_kwargs', 'get_nested_relation_kwargs', 'get_url_kwargs'
)

STRING_TYPES = (
    types.String, types.Text, types.Unicode, types.UnicodeText, types.CHAR,
    types.NCHAR, types.NVARCHAR, types.CLOB, types.TEXT
)


def get_detail_view_name(model):
    """
    Given a model class, return the view name to use for URL relationships
    that refer to instances of the model.
    """
    return '%(model_name)s-detail' % {
        'model_name': model.__tablename__.lower()
    }


def get_field_kwargs(field_name, model_field, model_class):
    """
    Creates a default instance of a basic non-relational field.
    """
    mapper = model_class.__mapper__

    kwargs = {}
    field_validator = mapper.validators.get(field_name, None)
    validator_kwarg = []

    # The following will only be used by ModelField classes.
    # Gets removed for everything else
    kwargs['model_field'] = model_field

    max_digits = getattr(model_field.type, 'precision', None)
    if max_digits is not None:
        kwargs['max_digits'] = max_digits

    decimal_places = getattr(model_field.type, 'scale', None)
    if decimal_places is not None:
        kwargs['decimal_places'] = decimal_places

    if model_field.primary_key:
        # If this field is read-only, then return early.
        # Further keyword arguments are not valid
        kwargs['read_only'] = True
        return kwargs

    if model_field.default or model_field.nullable:
        kwargs['required'] = False

    if model_field.nullable:
        kwargs['allow_null'] = True

    if model_field.nullable and isinstance(model_field.type, STRING_TYPES):
        kwargs['allow_blank'] = True

    if isinstance(model_field.type, types.Enum):
        # If this model field contains choices, then return early.
        # Further keyword arguments are not valid
        kwargs['choices'] = model_field.type.enums
        return kwargs

    # Ensure that max_length is passed explicitly as a keyword arg, rather
    # than as a validator
    max_length = getattr(model_field.type, 'length', None)
    if max_length is not None and isinstance(model_field.type, STRING_TYPES):
        kwargs['max_length'] = max_length

    if getattr(model_field, 'unique', False):
        validator = UniqueORMValidator(model_class, field_name)
        validator_kwarg.append(validator)

    # If described table has a validator, then wrap him and call him later
    # when it will be necessary
    if field_validator:
        validator_func = field_validator[0]
        wrapper = ORMFieldValidator(validator_func, model_field, field_name)
        validator_kwarg.append(wrapper)

    if validator_kwarg:
        kwargs['validators'] = validator_kwarg

    return kwargs


def get_relation_kwargs(field_name, relation_info):
    """
    Creates a default instance of a flat relational field.
    """
    model_field = relation_info.model_field
    related_model = relation_info.related_model
    to_many = relation_info.to_many
    to_field = relation_info.to_field
    has_through_model = relation_info.has_through_model

    kwargs = {
        'queryset': related_model,
        'view_name': get_detail_view_name(related_model)
    }

    if to_many:
        kwargs['many'] = True

    if isinstance(to_field, Column):
        kwargs['to_field'] = to_field

    if has_through_model:
        kwargs['read_only'] = True
        kwargs.pop('queryset', None)

    if model_field:

        field_verbose_name = model_field.argument.arg or None
        if field_verbose_name and needs_label(field_verbose_name, field_name):
            kwargs['label'] = capfirst(field_verbose_name)

        if kwargs.get('read_only', False):
            # If this field is read-only, then return early.
            # No further keyword arguments are valid
            return kwargs

    return kwargs


def get_nested_relation_kwargs(relation_info):
    kwargs = {'read_only': True}
    if relation_info.to_many:
        kwargs['many'] = True
    return kwargs


def get_url_kwargs(model_field):
    return {
        'view_name': get_detail_view_name(model_field)
    }



# -*- coding: utf-8 -*-
"""
Module, which provide classes and function for related and nested field.
"""
from copy import copy

from aiorest_ws.db.orm import relations
from aiorest_ws.db.orm.sqlalchemy.mixins import ORMSessionMixin, \
    SQLAlchemyMixin
from aiorest_ws.db.orm.sqlalchemy.exceptions import ObjectDoesNotExist
from aiorest_ws.db.orm.relations import PKOnlyObject
from aiorest_ws.exceptions import ImproperlyConfigured
from aiorest_ws.utils.fields import get_attribute

__all__ = (
    'ManyRelatedField', 'RelatedField', 'StringRelatedField',
    'PrimaryKeyRelatedField', 'HyperlinkedRelatedField',
    'HyperlinkedIdentityField'
)


class ManyRelatedField(relations.ManyRelatedField):
    """
    Relationships with `many=True` transparently get coerced into instead being
    a ManyRelatedField with a child relationship.
    The `ManyRelatedField` class is responsible for handling iterating through
    the values and passing each one to the child relationship.
    This class is treated as private API.
    You shouldn't generally need to be using this class directly yourself,
    and should instead simply set 'many=True' on the relationship.
    """
    def get_attribute(self, instance):
        object_state = instance._sa_instance_state
        if object_state.transient or object_state.pending:
            return []

        return get_attribute(instance, self.source_attrs)

    def __deepcopy__(self, memo):
        # Avoid to deepcopy instance attributes. Because some SQLAlchemy
        # objects (e.g. Query) doesn't override __deepcopy__ methods, it leads
        # to some errors in runtime (e.g. maximum recursion limit)
        return copy(self)


class RelatedField(ORMSessionMixin, relations.RelatedField):
    many_related_field = ManyRelatedField

    def __deepcopy__(self, memo):
        # Avoid to deepcopy instance attributes. Because some SQLAlchemy
        # objects (e.g. Query) doesn't override __deepcopy__ methods, it leads
        # to some errors in runtime (e.g. maximum recursion limit)
        return copy(self)

    def get_attribute(self, instance):
        if self.use_pk_only_optimization() and self.source_attrs:
            # Optimized case, return a mock object only containing the
            # pk attribute
            try:
                instance = get_attribute(instance, self.source_attrs[:-1])
                value = getattr(instance, self.source_attrs[-1])
                return PKOnlyObject(pk=value)
            except AttributeError:
                pass

        # Standard case, return the object instance
        return get_attribute(instance, self.source_attrs)


class StringRelatedField(relations.StringRelatedField, RelatedField):
    pass


class PrimaryKeyRelatedField(relations.PrimaryKeyRelatedField,
                             SQLAlchemyMixin, RelatedField):

    def to_internal_value(self, data):
        if self.pk_field is not None:
            data = self.pk_field.to_internal_value(data)

        query = self.get_queryset()
        try:
            filter_args = self._get_filter_args(query, data)
            obj = query.filter(*filter_args).first()
            if not obj:
                raise ObjectDoesNotExist()
            return obj
        except ObjectDoesNotExist:
            self.raise_error('does_not_exist', pk_value=data)
        except (TypeError, ValueError):
            self.raise_error('incorrect_type', data_type=type(data).__name__)
        finally:
            query.session.close()

    def to_representation(self, value):
        value = value.pk if isinstance(value, PKOnlyObject) else value
        object_pk = self._get_object_pk(value)
        if self.pk_field is not None:
            return self.pk_field.to_representation(object_pk)
        return object_pk


class HyperlinkedRelatedField(relations.HyperlinkedRelatedField,
                              SQLAlchemyMixin, RelatedField):
    lookup_field = 'id'  # or `pk`

    def use_pk_only_optimization(self):
        return self.lookup_field in ('id', 'pk')

    def get_object(self, view_name, view_args, view_kwargs):
        """
        Return the object corresponding to a matched URL.
        Takes the matched URL conf arguments, and should return an
        object instance, or raise an `ObjectDoesNotExist` exception.
        """
        query = self.get_queryset()
        try:
            lookup_value = view_kwargs[self.lookup_url_kwarg]
            lookup_kwargs = {self.lookup_field: lookup_value}
            filter_args = self._get_filter_args(query, lookup_kwargs)
            obj = query.filter(*filter_args).first()
            if not obj:
                raise ObjectDoesNotExist()
            return obj
        except ObjectDoesNotExist:
            self.raise_error('does_not_exist')
        except KeyError:
            raise ImproperlyConfigured(
                "Missing primary key in the endpoint path. For fixing it just "
                "specify for a requested endpoint URL with included "
                "`{field_name}` parameter in the path or override"
                "`lookup_url_kwarg` in the constructor for the concrete field."
                .format(field_name=self.lookup_url_kwarg)
            )
        except (TypeError, ValueError):
            self.raise_error(
                'incorrect_type', data_type=type(view_kwargs).__name__
            )
        finally:
            query.session.close()

    def is_saved_in_database(self, obj):
        """
        Return the boolean value, which let us to understand whether saved
        object in a database or not.
        """
        object_state = obj._sa_instance_state
        if object_state.transient or object_state.pending:
            return False
        return True

    def get_lookup_value(self, obj):
        """
        Return a tuple of object lookup values, which are will be using for
        reverse operation.
        """
        object_pk = self._get_object_pk(obj)
        if isinstance(object_pk, dict):
            return object_pk.values()
        return object_pk,


class HyperlinkedIdentityField(relations.HyperlinkedIdentityField,
                               HyperlinkedRelatedField):
    pass



# -*- coding: utf-8 -*-
"""
List, model and hyperlinked serializer classes for SQLAlchemy ORM.

As you can see there, we are inherited from all base classes and implement
logic according to the work of SQLAlchemy ORM. For the most situations using
ModelSerializer class will be enough.
"""
from aiorest_ws.conf import settings
from aiorest_ws.db.orm.abstract import empty
from aiorest_ws.db.orm.serializers import \
    ListSerializer as BaseListSerializer, \
    ModelSerializer as BaseModelSerializer, HyperlinkedModelSerializerMixin, \
    raise_errors_on_nested_writes
from aiorest_ws.db.orm.sqlalchemy import model_meta
from aiorest_ws.db.orm.sqlalchemy.field_mapping import get_field_kwargs, \
    get_relation_kwargs, get_nested_relation_kwargs, get_url_kwargs
from aiorest_ws.db.orm.exceptions import ValidationError
from aiorest_ws.utils.field_mapping import ClassLookupDict

from sqlalchemy import types, Column
from sqlalchemy.sql.schema import ColumnCollectionConstraint
from sqlalchemy.orm.interfaces import MANYTOMANY, MANYTOONE, ONETOMANY
from sqlalchemy.orm.properties import ColumnProperty
from sqlalchemy.dialects import postgresql

from aiorest_ws.db.orm.sqlalchemy.fields import *  # NOQA
from aiorest_ws.db.orm.sqlalchemy.relations import *  # NOQA

__all__ = (
    'get_validation_error_detail', 'ListSerializer', 'ModelSerializer',
    'HyperlinkedModelSerializer',
)


def get_validation_error_detail(exc):
    assert isinstance(exc, (ValidationError, AssertionError))

    if isinstance(exc, AssertionError):
        return {settings.REST_CONFIG['NON_FIELD_ERRORS_KEY']: exc.args}
    elif isinstance(exc.detail, dict):
        # If errors may be a dict we use the standard {key: list of values}.
        # Here we ensure that all the values are *lists* of errors.
        return {
            key: value if isinstance(value, (list, dict)) else [value]
            for key, value in exc.detail.items()
        }
    elif isinstance(exc.detail, list):
        # Errors raised as a list are non-field errors.
        return {settings.REST_CONFIG['NON_FIELD_ERRORS_KEY']: exc.detail}
    # Errors raised as a string are non-field errors.
    return {settings.REST_CONFIG['NON_FIELD_ERRORS_KEY']: [exc.detail]}


class ListSerializer(BaseListSerializer):

    def run_validation(self, data=empty):
        """
        We override the default `run_validation`, because the validation
        performed by validators and the `.validate()` method should
        be coerced into an error dictionary with a 'non_fields_error' key.
        """
        (is_empty_value, data) = self.validate_empty_values(data)
        if is_empty_value:
            return data

        value = self.to_internal_value(data)
        try:
            self.run_validators(value)
            value = self.validate(value)
            assert value is not None, '.validate() should return the validated data'  # NOQA
        except (ValidationError, AssertionError) as exc:
            raise ValidationError(detail=get_validation_error_detail(exc))

        return value


class ModelSerializer(BaseModelSerializer):
    """
    Base serializer for SQLAlchemy models.

    This class automatically generate "scheme" for further serializing and
    de-serializing data, according to used model, which is specified by user.
    For indicated fields by programmer, ModelSerializer will not apply "scheme
    construction rule". They are will have skipped while processing. Otherwise,
    for every field which is not pre-defined by user, will be found the most
    suitable and compatible field class.
    """
    serializer_field_mapping = {
        # Generic types
        types.BigInteger: BigIntegerField,
        types.Binary: LargeBinaryField,
        types.Boolean: BooleanField,
        types.Date: DateField,
        types.DateTime: DateTimeField,
        types.Enum: EnumField,
        types.Float: FloatField,
        types.Integer: IntegerField,
        types.Interval: IntervalField,
        types.LargeBinary: LargeBinaryField,
        types.MatchType: BooleanField,
        types.Numeric: DecimalField,
        types.PickleType: PickleField,
        types.SmallInteger: SmallIntegerField,
        types.String: CharField,
        types.Text: CharField,
        types.Time: TimeField,
        types.Unicode: CharField,
        types.UnicodeText: CharField,
        # SQL standard and multiple vendor types
        types.BIGINT: BigIntegerField,
        types.BLOB: LargeBinaryField,
        types.BOOLEAN: BooleanField,
        types.CHAR: CharField,
        types.CLOB: CharField,
        types.DATE: DateField,
        types.DATETIME: DateTimeField,
        types.DECIMAL: DecimalField,
        types.FLOAT: FloatField,
        types.INT: IntegerField,
        types.INTEGER: IntegerField,
        types.NCHAR: CharField,
        types.NVARCHAR: CharField,
        types.NUMERIC: DecimalField,
        types.REAL: FloatField,
        types.SMALLINT: SmallIntegerField,
        types.TEXT: CharField,
        types.TIMESTAMP: DateTimeField,
        types.VARBINARY: LargeBinaryField,
        types.VARCHAR: CharField,
        # Special SQLAlchemy types
        postgresql.ARRAY: ListField,
        postgresql.HSTORE: HStoreField,
        # For any other field types (include custom)
        types.TypeEngine: ModelField
    }
    serializer_related_field = PrimaryKeyRelatedField
    serializer_related_to_field = None
    serializer_url_field = HyperlinkedIdentityField
    serializer_choice_field = EnumField
    default_list_serializer = ListSerializer

    def is_abstract_model(self, model):
        """
        Check the passed model is abstract.
        """
        return model_meta.is_abstract_model(model)

    def get_field_info(self, model):
        """
        Get metadata about field in the passed model.
        """
        return model_meta.get_field_info(model)

    # Default `create` and `update` behavior
    def create(self, validated_data):
        """
        We have a bit of extra checking around this in order to provide
        descriptive messages when something goes wrong, but this method is
        essentially just:

            instance = ExampleModel(**validated_data)
            session.add(instance)
            session.commit()
            return object_to_dict(instance)

        If there are many to many fields present on the instance then they
        cannot be set until the model is instantiated, in which case the
        implementation is like so:

            example_relationship = validated_data.pop('example_relationship')
            instance = ExampleModel(**validated_data)
            instance.example_relationship = example_relationship
            session.add(instance)
            session.commit()
            return object_to_dict(instance)

        The default implementation also does not handle nested relationships.
        If you want to support writable nested relationships you'll need
        to write an explicit `.create()` method.
        """
        raise_errors_on_nested_writes('create', self, validated_data)

        ModelClass = self.Meta.model
        # Remove relationship instances from validated_data.
        # They are not valid arguments to the default `.create()` method,
        # as they require that the instance has already been saved
        relations = model_meta.get_relations_data(ModelClass, validated_data)

        session = settings.SQLALCHEMY_SESSION(expire_on_commit=False)
        try:
            instance = ModelClass(**validated_data)
            # After creating instance of ModelClass, just append all data,
            # which are removed earlier
            if relations:
                session.enable_relationship_loading(instance)
                for field_name, value in relations.items():
                    setattr(instance, field_name, value)

            session.add(instance)
            session.commit()
        except TypeError as exc:
            msg = (
                'Got a `TypeError` when calling `%s()`. '
                'This may be because you have a writable field on the '
                'serializer class that is not a valid argument to '
                '`%s.objects.create()`. You may need to make the field '
                'read-only, or override the %s.create() method to handle '
                'this correctly.\nOriginal exception text was: %s.' %
                (
                    ModelClass.__name__,
                    ModelClass.__name__,
                    self.__class__.__name__,
                    exc
                )
            )
            raise TypeError(msg)
        finally:
            session.close()

        return instance

    def update(self, instance, validated_data):
        raise_errors_on_nested_writes('update', self, validated_data)

        ModelClass = self.Meta.model
        relations = model_meta.get_relations_data(ModelClass, validated_data)

        # Generate filter for getting existing object once more. Earlier,
        # We generate query to the database, because we can get relationship
        # objects, which attached to the existing object. But can't use this
        # instance further. To avoid issues with it, just get object once more
        filter_args = (
            getattr(ModelClass, field) == getattr(instance, field)
            for field in model_meta.model_pk(ModelClass)
        )

        # Simply set each attribute on the instance, and then save it.
        # Note that unlike `.create()` we don't need to treat many-to-many
        # relationships as being a special case. During updates we already
        # have an instance pk for the relationships to be associated with
        session = settings.SQLALCHEMY_SESSION(expire_on_commit=False)
        try:
            instance = session.query(ModelClass).filter(*filter_args).first()
            if not instance:
                raise ValidationError("Object does not exist.")

            for field_name, value in validated_data.items():
                setattr(instance, field_name, value)

            # Update relation objects
            if relations:
                session.enable_relationship_loading(instance)
                for field_name, value in relations.items():
                    setattr(instance, field_name, [])
                    session.refresh(instance)

                    if value:
                        setattr(instance, field_name, value)

            session.commit()
        finally:
            session.close()

        return instance

    def run_validation(self, data=empty):
        (is_empty_value, data) = self.validate_empty_values(data)
        if is_empty_value:
            return data

        value = self.to_internal_value(data)
        try:
            self.run_validators(value)
            value = self.validate(value)
            assert value is not None, '.validate() should return the validated data'  # NOQA
        except (ValidationError, AssertionError) as exc:
            raise ValidationError(detail=get_validation_error_detail(exc))

        return value

    def get_default_field_names(self, declared_fields, model_info):
        """
        Return the default list of field names that will be used if the
        `Meta.fields` option is not specified.
        """
        return list(set(
            list(dict(model_info.pk)) +
            list(declared_fields.keys()) +
            list(model_info.fields.keys()) +
            list(model_info.forward_relations.keys())
        ))

    def _get_model_fields(self, field_names, declared_fields, extra_kwargs):
        """
        Returns all the model fields that are being mapped to by fields
        on the serializer class.
        Returned as a dict of 'model field name' -> 'model field'.
        Used internally by `get_uniqueness_field_options`.
        """
        model = getattr(self.Meta, 'model')
        model_fields = {}

        for field_name in field_names:
            if field_name in declared_fields:
                # If the field is declared on the serializer
                field = declared_fields[field_name]
                source = field.source or field_name
            else:
                try:
                    source = extra_kwargs[field_name]['source']
                except KeyError:
                    source = field_name

            if '.' in source or source == '*':
                # Model fields will always have a simple source mapping,
                # they can't be nested attribute lookups
                continue

            self._bind_field(model, source, model_fields)

        return model_fields

    def _get_unique_constraint_names(self, model, model_fields, field_names):
        """
        Return a set of field names, for each column unique constraint.
        Used internally by `get_uniqueness_extra_kwargs`.
        """
        unique_constraint_names = set()

        # Iterate over all model field and include every field, which
        # marked have value `True` for `unique` attribute
        for field_name in model_fields.keys():
            column = getattr(model, field_name)
            unique_constraint_names |= {
                column.name
                for column in column.property.columns
                if column.unique
            }

        unique_constraint_names -= {None}
        return unique_constraint_names

    def _get_unique_together_constraints(self, model, model_fields, field_names):  # NOQA
        """
        Return a set of field names for a multiple unique constraints.
        Used internally by `get_uniqueness_extra_kwargs`.
        """
        unique_constraint_names = set()

        # Scan tables for passed model and include every unique field
        # as at the previous sub-step. Also, there we try to find special
        # unique constrains (e.g. CheckConstraint, ForeignKeyConstraint,
        # PrimaryKeyConstraint, UniqueConstraint)
        for constraint in model.__table__.constraints:
            if isinstance(constraint, ColumnCollectionConstraint):
                unique_constraint_names |= set(constraint.columns.keys())

        return unique_constraint_names

    def _get_unique_field(self, model, unique_field_name):
        """
        Return a field by his name from a model.
        Used internally by `get_uniqueness_extra_kwargs`.
        """
        return model.__mapper__.columns[unique_field_name]

    def _get_default_field_value(self, unique_constraint_field):
        """
        Return a default value for a passed field.
        Used internally by `get_uniqueness_extra_kwargs`.
        """
        default = empty

        if unique_constraint_field.default:
            default = unique_constraint_field.default
        elif unique_constraint_field.onupdate:
            default = unique_constraint_field.onupdate

        return default

    def build_field(self, field_name, info, model_class, nested_depth):
        """
        Create regular model fields.
        """
        if field_name in info.fields_and_pk:
            model_field = info.fields_and_pk[field_name]
            return self.build_standard_field(field_name, model_field, model_class)  # NOQA

        elif field_name in info.relations:
            relation_info = info.relations[field_name]
            if not nested_depth:
                return self.build_relational_field(field_name, relation_info)
            else:
                return self.build_nested_field(field_name, relation_info, nested_depth)  # NOQA

        elif hasattr(model_class, field_name):
            return self.build_property_field(field_name, model_class)

        elif field_name == self.url_field_name:
            return self.build_url_field(field_name, model_class)

        return self.build_unknown_field(field_name, model_class)

    def build_standard_field(self, *args, **kwargs):
        """
        Create regular model fields.
        """
        field_name, model_field, model_class = args
        field_mapping = ClassLookupDict(self.serializer_field_mapping)

        field_class = field_mapping[model_field.type]
        field_kwargs = get_field_kwargs(field_name, model_field, model_class)

        if 'choices' in field_kwargs:
            # Fields with choices get coerced into `ChoiceField`
            # instead of using their regular typed field
            field_class = self.serializer_choice_field
            # Some model fields may introduce kwargs that would not be valid
            # for the choice field. We need to strip these out.
            # Eg. Column('race', Enum('one', 'two', 'three'))
            valid_kwargs = {
                'read_only', 'write_only',
                'required', 'default', 'initial', 'source',
                'label', 'help_text', 'style',
                'error_messages', 'validators', 'allow_null', 'allow_blank',
                'choices'
            }
            for key in list(field_kwargs.keys()):
                if key not in valid_kwargs:
                    field_kwargs.pop(key)

        if not issubclass(field_class, ModelField):
            # `model_field` is only valid for the fallback case of
            # `ModelField`, which is used when no other typed field
            # matched to the model field
            field_kwargs.pop('model_field', None)

        if not issubclass(field_class, CharField) and \
                not issubclass(field_class, EnumField):
            # `allow_blank` is only valid for textual fields
            field_kwargs.pop('allow_blank', None)

        if isinstance(model_field.type, postgresql.ARRAY):
            # Populate the `child` argument on `ListField` instances generated
            # for the PostgreSQL specific `ArrayField`
            child_field_name = '%s.%s' % (field_name, 'child')
            child_model_field = model_field.type.item_type
            # Because our model_field, which passed to the build_standard_field
            # function is a Column, then wrap this sub-field and invoke this
            # method one more time
            child_field_class, child_field_kwargs = self.build_standard_field(
                child_field_name, Column(child_model_field), model_class
            )
            field_kwargs['child'] = child_field_class(**child_field_kwargs)

        return field_class, field_kwargs

    def build_relational_field(self, *args, **kwargs):
        """
        Create fields for forward and reverse relationships.
        """
        field_name, relation_info = args

        field_class = self.serializer_related_field
        field_kwargs = get_relation_kwargs(field_name, relation_info)

        relation_type = relation_info.model_field.direction
        if relation_type in (MANYTOMANY, MANYTOONE, ONETOMANY):
            field_kwargs['many'] = True

        # `to_field` is only valid for slug fields, which aren't
        # supported by SQLAlchemy ORM by default
        field_kwargs.pop('to_field', None)

        # `view_name` is only valid for hyperlinked relationships
        if not issubclass(field_class, HyperlinkedRelatedField):
            field_kwargs.pop('view_name', None)

        return field_class, field_kwargs

    def build_nested_field(self, *args, **kwargs):
        """
        Create nested fields for forward and reverse relationships.
        """
        field_name, relation_info, nested_depth = args

        class NestedSerializer(ModelSerializer):
            class Meta:
                model = relation_info.related_model
                depth = nested_depth - 1

        field_class = NestedSerializer
        field_kwargs = get_nested_relation_kwargs(relation_info)

        return field_class, field_kwargs

    def build_property_field(self, *args, **kwargs):
        """
        Create a read only field for model methods and properties.
        """
        field_class = ReadOnlyField
        field_kwargs = {}

        return field_class, field_kwargs

    def build_url_field(self, field_name, model_class):
        """
        Create a field representing the object's own URL.
        """
        field_class = self.serializer_url_field
        field_kwargs = get_url_kwargs(model_class)

        return field_class, field_kwargs

    def _bind_field(self, model, source, model_fields):
        try:
            column = getattr(model, source, None)
            if column and isinstance(column.property, ColumnProperty):
                model_fields[source] = type(column.property.columns[0].type)
        except (AttributeError, IndexError):
            pass


class HyperlinkedModelSerializer(HyperlinkedModelSerializerMixin,
                                 ModelSerializer):
    serializer_related_field = HyperlinkedRelatedField

    def build_nested_field(self, field_name, relation_info, nested_depth):
        """
        Create nested fields for forward and reverse relationships.
        """
        class NestedSerializer(HyperlinkedModelSerializer):
            class Meta:
                model = relation_info.related_model
                depth = nested_depth - 1

        field_class = NestedSerializer
        field_kwargs = get_nested_relation_kwargs(relation_info)

        return field_class, field_kwargs



# -*- coding: utf-8 -*-
"""
Functions and decorators used for modify classes on the fly.
"""
__all__ = ('add_property', )


def add_property(instance, field_name, value):
    """
    Append property to the current instance of class.

    :param instance: modified object.
    :param field_name: attribute name in class.
    :param value: value, which used for initialize the field_name.
    """
    def get_protected_field(self):
        return getattr(self, protected_field_name)

    cls = type(instance)
    protected_field_name = '_{0}'.format(field_name)
    property_func = get_protected_field
    setattr(instance, protected_field_name, value)
    setattr(cls, field_name, property(property_func))



# -*- coding: utf-8 -*-
"""
Custom functions and classes, which help working with the command line.
"""
from argparse import ArgumentParser

__all__ = ('CommandLine', )


class CommandLine(object):
    """
    Wrapper over ArgumentParser class for working with command line.
    """
    options = ArgumentParser()

    def define(self, name, default=None, help=None, type=None):
        """
        Defines an option in the global namespace.

        Note: already defined argument or option has been ignored and not
              appended again!

        :param name: used argument/option in command line (e.c. -f or --foo).
        :param default: used value for argument or option if not specified.
        :param help: full description for option, which used when invoked
                     program with -h flag.
        :param type: preferred type for option.
        """
        if name not in self.options._option_string_actions.keys():
            self.options.add_argument(name, default=default, help=help,
                                      type=type)

    def parse_command_line(self):
        """
        Parse options from the command line.
        """
        return self.options.parse_args()



# -*- coding: utf-8 -*-
"""
Utility exceptions for resolve/reverse URLs purposes.
"""
__all__ = ('NoMatch', 'NoReverseMatch')


class NoMatch(Exception):
    pass


class NoReverseMatch(Exception):
    pass



# -*- coding: utf-8 -*-
"""
WebSocket status codes and functions for work with them.

For more details check the link below:
    https://tools.ietf.org/html/rfc6455
"""
__all__ = (
    'WS_NORMAL', 'WS_GOING_AWAY', 'WS_PROTOCOL_ERROR',
    'WS_DATA_CANNOT_ACCEPT', 'WS_RESERVED', 'WS_NO_STATUS_CODE',
    'WS_CLOSED_ABNORMALLY', 'WS_MESSAGE_NOT_CONSISTENT',
    'WS_MESSAGE_VIOLATE_POLICY', 'WS_MESSAGE_TOO_BIG',
    'WS_SERVER_DIDNT_RETURN_EXTENSIONS', 'WS_UNEXPECTED_CONDITION',
    'WS_FAILURE_TLS',
    'is_not_used', 'is_reserved', 'is_library', 'is_private',
)


WS_NORMAL = 1000
WS_GOING_AWAY = 1001
WS_PROTOCOL_ERROR = 1002
WS_DATA_CANNOT_ACCEPT = 1003
WS_RESERVED = 1004
WS_NO_STATUS_CODE = 1005
WS_CLOSED_ABNORMALLY = 1006
WS_MESSAGE_NOT_CONSISTENT = 1007
WS_MESSAGE_VIOLATE_POLICY = 1008
WS_MESSAGE_TOO_BIG = 1009
WS_SERVER_DIDNT_RETURN_EXTENSIONS = 1010
WS_UNEXPECTED_CONDITION = 1011
WS_FAILURE_TLS = 1015


def is_not_used(code):
    """
    Checking code, that is unused.

    :param code: integer value.
    """
    return 0 <= code <= 999


def is_reserved(code):
    """
    Checking code, that is reserved.

    :param code: integer value.
    """
    return 1000 <= code <= 2999


def is_library(code):
    """
    Checking code, that is value, used by libraries.

    :param code: integer value.
    """
    return 3000 <= code <= 3999


def is_private(code):
    """
    Checking code, that is private code.

    :param code: integer value.
    """
    return 4000 <= code <= 4999



# -*- coding: utf-8 -*-
"""
This module implements the central application object.
"""
import asyncio
import ssl
from time import gmtime, strftime

from aiorest_ws.__init__ import __version__
from aiorest_ws.request import RequestHandlerFactory, RequestHandlerProtocol
from aiorest_ws.validators import check_and_set_subclass
from aiorest_ws.utils.websocket import deflate_offer_accept as accept
from aiorest_ws.urls.base import set_urlconf

__all__ = ('Application', )


class Application(object):
    """
    Main application of aiorest-ws framework.
    """

    _factory = RequestHandlerFactory
    _protocol = RequestHandlerProtocol
    _certificate = None
    _key = None
    _middlewares = []

    def __init__(self, *args, **options):
        """
        Initialization of Application instance.
        """
        super(Application, self).__init__()
        self.factory = options.get('factory')
        self.protocol = options.get('protocol')
        self.certificate = options.get('certificate')
        self.key = options.get('key')

        middleware_classes = options.get('middlewares', ())
        for middleware in middleware_classes:
            self._middlewares.append(middleware())

    @property
    def factory(self):
        """
        Get factory class.
        """
        return self._factory

    @factory.setter
    def factory(self, factory):
        """
        Set factory class.

        :param factory: subclass of RequestHandlerFactory.
        """
        if factory:
            check_and_set_subclass(self, '_factory', factory,
                                   RequestHandlerFactory)

    @property
    def middlewares(self):
        """
        Get list of used middlewares.
        """
        return self._middlewares

    @property
    def protocol(self):
        """
        Get protocol class.
        """
        return self._protocol

    @protocol.setter
    def protocol(self, protocol):
        """
        Set protocol class.

        :param factory: subclass of RequestHandlerProtocol.
        """
        if protocol:
            check_and_set_subclass(self, '_protocol', protocol,
                                   RequestHandlerProtocol)

    @property
    def certificate(self):
        """
        Get filepath to certificate.
        """
        return self._certificate

    @certificate.setter
    def certificate(self, certificate):
        """
        Setter for certificate.

        :param certificate: path to certificate file.
        """
        self._certificate = certificate

    @property
    def key(self):
        """
        Get private key for certificate.
        """
        return self._key

    @key.setter
    def key(self, key):
        """
        Set private key for certificate.

        :param key: private key for certificate.
        """
        self._key = key

    @property
    def url(self):
        """
        Get url to WebSocket REST API.
        """
        return "wss://{0}:{1}/{2}" if self.isSecure else "ws://{0}:{1}/{2}"

    @property
    def isSecure(self):
        """
        Property, which help us to understand, use SSL or not.
        """
        return self.certificate and self.key

    def _get_ssl_context(self):
        """
        Generating SSL context for asyncio loop.
        """
        if self.isSecure:
            ssl_context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
            ssl_context.load_cert_chain(self.certificate, self.key)
        else:
            ssl_context = None
        return ssl_context

    def _init_factory(self, url, **options):
        """
        Create a factory instance.
        """
        log_level = options.get('log_level', 'info')
        factory = self.factory(url)
        factory.protocol = self.protocol
        factory.log._set_log_level(log_level)
        return factory

    def _enable_compressing(self, factory, **options):
        """
        Set compression message for factory, if defined.
        """
        compress = options.get('compress', False)
        accept_function = options.get('accept_function', accept)
        protocol_options = {"perMessageCompressionAccept": accept_function}

        if compress:
            factory.setProtocolOptions(**protocol_options)

    def _set_factory_router(self, factory, **options):
        """
        Set users router for factory, if defined.
        """
        router = options.get('router', None)
        assert router, "Argument `router` must be defined for the Application."

        factory.router = router
        factory.router._middlewares = self.middlewares

    def _init_urlconf(self, factory, url, **options):
        """
        Initialize urlconf thread variable.
        """
        data = {
            'path': url.strip('/'),
            'urls': factory.router._urls,
            'routes': factory.router._routes
        }
        set_urlconf(data)

    def generate_factory(self, url, **options):
        """
        Create and initialize factory instance.
        """
        factory = self._init_factory(url, **options)
        self._enable_compressing(factory, **options)
        self._set_factory_router(factory, **options)
        self._init_urlconf(factory, url, **options)
        return factory

    def generate_url(self, host, port, path=''):
        """
        Generate URL to application.
        """
        return self.url.format(host, port, path)

    def run(self, **options):
        """
        Create and start web server with some IP and PORT.

        :param options: parameters, which can be used for configuration
                        of the Application.
        """
        host = options.get('host', '127.0.0.1')
        port = options.get('port', 8080)
        path = options.get('path', '')
        url = self.generate_url(host, port, path)

        factory = self.generate_factory(url, **options)
        ssl_context = self._get_ssl_context()

        loop = asyncio.get_event_loop()
        server_coroutine = loop.create_server(
            factory, host, port, ssl=ssl_context
        )
        server = loop.run_until_complete(server_coroutine)

        print(strftime("%d %b, %Y - %X", gmtime()))
        print("aiorest-ws version {0}".format(__version__))
        print("Server started at {0}".format(url))
        print("Quit the server with CONTROL-C.")

        try:
            loop.run_forever()
        except KeyboardInterrupt:
            pass
        finally:
            server.close()
            loop.close()



# -*- coding: utf-8 -*-
"""
Default settings for configuration behaviour of aiorest-ws framework.
"""

# -----------------------------------------------
#  Basic settings
# -----------------------------------------------
# Default datetime input and output formats
ISO_8601 = 'iso-8601'
DATE_FORMAT = ISO_8601
DATE_INPUT_FORMATS = (ISO_8601, )

DATETIME_FORMAT = ISO_8601
DATETIME_INPUT_FORMATS = (ISO_8601, )

TIME_FORMAT = ISO_8601
TIME_INPUT_FORMATS = (ISO_8601,)

# Local time zone for this installation. All choices can be found here:
# https://en.wikipedia.org/wiki/List_of_tz_zones_by_name (although not all
# systems may support all possibilities). When USE_TZ is True, this is
# interpreted as the default user time zone
TIME_ZONE = 'America/Chicago'

# If you set this to True, Django will use timezone-aware datetimes
USE_TZ = False

# Encoding charset for string, files, etc.
DEFAULT_ENCODING = 'utf-8'

UNICODE_JSON = True
COMPACT_JSON = True
COERCE_DECIMAL_TO_STRING = True
UPLOADED_FILES_USE_URL = True

# -----------------------------------------------
#  Database
# -----------------------------------------------
# This dictionary very useful, cause we can customize it and set required
# drivers, paths, credentials, for every user database
DATABASES = {}
USE_ORM_ENGINE = False

# SQLAlchemy ORM variables
SQLALCHEMY_ENGINE = None
SQLALCHEMY_SESSION = None

# -----------------------------------------------
#  REST configuration
# -----------------------------------------------
REST_CONFIG = {
    # Exception handling
    'NON_FIELD_ERRORS_KEY': 'non_field_errors',

    # Hyperlink settings
    'URL_FIELD_NAME': 'url',
}

# -----------------------------------------------
#  Middleware
# -----------------------------------------------
# List of middleware classes, which assigned for the main router. All
# middlewares will be used in the order of enumeration. Keep in mind, when
# use this feature
MIDDLEWARE_CLASSES = ()

# -----------------------------------------------
# Logging
# -----------------------------------------------
DEFAULT_LOGGING_SETTINGS = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': "[%(asctime)s] [%(levelname)s] %(message)s",
            'datefmt': "%d/%b/%Y %H:%M:%S"
        },
        'verbose': {
            'format': "[%(asctime)s] [%(levelname)s] [%(name)s:%(lineno)s] "
                      "%(message)s",
            'datefmt': "%d/%b/%Y %H:%M:%S"
        },
    },
    'handlers': {
        'default': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'standard',
        },
        'debug': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'aiorest-ws': {
            'handlers': ['default', ],
            'level': 'INFO',
            'propagate': False
        },
        'debug': {
            'handlers': ['debug', ],
            'level': 'DEBUG',
            'propagate': False
        }
    }
}



# -*- coding: utf-8 -*-
"""
User model for authentication.
"""
from sqlite3 import OperationalError

from aiorest_ws.auth.user.abstractions import User
from aiorest_ws.auth.user.utils import generate_password_hash
from aiorest_ws.auth.user.exceptions import RequiredModelFieldsNotDefined, \
    SearchCriteriaRequired, NotEnoughArguments
from aiorest_ws.auth.user.utils import SQL_CREATE_USER_TABLE, \
    SQL_CREATE_TOKEN_FOREIGN_KEY, SQL_USER_GET, SQL_USER_GET_BY_USERNAME, \
    SQL_USER_ADD, USER_MODEL_FIELDS, USER_MODEL_FIELDS_WITHOUT_PK, \
    SQL_USER_GET_WITH_ID, construct_update_sql, convert_user_raw_data_to_dict
from aiorest_ws.conf import settings
from aiorest_ws.db.backends.sqlite3.constants import IN_MEMORY
from aiorest_ws.db.backends.sqlite3.managers import SQLiteManager
from aiorest_ws.log import logger

__all__ = ('UserSQLiteModel', )


class UserSQLiteModel(User):
    """
    SQLite user model.
    """
    db_manager = SQLiteManager

    def __init__(self):
        super(UserSQLiteModel, self).__init__()
        if settings.DATABASES:
            db_path = settings.DATABASES['default']['name']
            self.db_manager = settings.DATABASES['default']['manager']
        else:
            db_path = IN_MEMORY
            self.db_manager = self.db_manager(name=db_path)

        if db_path == IN_MEMORY:
            self.__create_models()

    def __create_models(self):
        """
        Create user model and append foreign key into token table.
        """
        try:
            self.db_manager.execute_script(SQL_CREATE_USER_TABLE)
            self.db_manager.execute_script(SQL_CREATE_TOKEN_FOREIGN_KEY)
        # This exception taken only in the case, when `user_id` foreign
        # keys already created. We didn't have any opportunity to check
        # existing column via SQL, because SQL syntax of SQLite is reduced
        except OperationalError:
            pass

    def __user_defined_fields(self, init_data):
        """
        Define fields in which changed data by the user.

        :param init_data: data, taken from the user.
        """
        overridden_fields = set(self.fields) & set(init_data.keys())
        user_defined_fields = {
            key: value
            for key, value in init_data.items()
            if key in overridden_fields
        }
        return user_defined_fields

    @property
    def fields(self):
        """
        Get list of fields with primary key.
        """
        return USER_MODEL_FIELDS

    @property
    def fields_without_pk(self):
        """
        Get list of fields without primary key.
        """
        return USER_MODEL_FIELDS_WITHOUT_PK

    def create_user(self, *args, **kwargs):
        """
        Create user in the database.

        :param args: tuple of arguments.
        :param kwargs: dictionary, where key is filled field of user model.
        """
        if 'username' not in kwargs or 'password' not in kwargs:
            raise RequiredModelFieldsNotDefined(
                "Username and password fields are required"
            )

        kwargs['password'] = generate_password_hash(kwargs['password'])

        user_defined_fields = self.__user_defined_fields(kwargs)
        default_user_data = {
            'first_name': '',
            'last_name': '',
            'is_active': True,
            'is_superuser': False,
            'is_staff': False,
            'is_user': False,
        }

        default_user_data.update(user_defined_fields)
        user_data = [default_user_data[key] for key in self.fields_without_pk]

        try:
            self.db_manager.execute_sql(SQL_USER_ADD, user_data)
        except OperationalError as exc:
            logger.error(exc)

    def update_user(self, *args, **kwargs):
        """
        Update user row in the database.

        :param args: tuple of arguments.
        :param kwargs: dictionary, where key is updated field of user model.
        """
        username = kwargs.pop('username', None)
        if not username:
            raise SearchCriteriaRequired("Username for WHEN statement is required.")  # NOQA

        if len(kwargs) < 1:
            raise NotEnoughArguments()

        if 'password' in kwargs.keys():
            kwargs['password'] = generate_password_hash(kwargs['password'])

        updated_fields = self.__user_defined_fields(kwargs)
        update_query, query_args = construct_update_sql(**updated_fields)
        query_args.append(username)
        try:
            self.db_manager.execute_sql(update_query, query_args)
        except OperationalError as exc:
            logger.error(exc)

    def get_user_by_username(self, username, with_id=False):
        """
        Get user by his username from the database.

        :param username: username as a string.
        :param with_id: boolean flag, which means necessity to append to the
                        result object primary key of database row or not.
        """
        try:
            if with_id:
                sql = SQL_USER_GET_WITH_ID
            else:
                sql = SQL_USER_GET_BY_USERNAME

            user_row = self.db_manager.execute_sql(sql, (username, )).fetchone()  # NOQA
            if user_row:
                user_data = convert_user_raw_data_to_dict(user_row, with_id)
            else:
                user_data = {}
        except OperationalError as exc:
            logger.error(exc)
            user_data = {}
        return User(**user_data)

    def get_user_by_token(self, token):
        """
        Get user object from the database, based on the his token.

        :param token: passed token as a dictionary object.
        """
        user_id = token['user_id']
        try:
            user_row = self.db_manager.execute_sql(
                SQL_USER_GET, (user_id, )
            ).fetchone()
            if user_row:
                user_data = convert_user_raw_data_to_dict(user_row)
            else:
                user_data = {}
        except OperationalError as exc:
            logger.error(exc)
            user_data = {}
        return User(**user_data)



# -*- coding: utf-8 -*-
"""
This module provide a function and class-based views and can be used
with aiorest-ws routers.
"""
from aiorest_ws.exceptions import IncorrectMethodNameType, \
    InvalidRenderer, NotSpecifiedHandler, NotSpecifiedMethodName
from aiorest_ws.renderers import JSONRenderer

__all__ = ('http_methods', 'View', 'MethodViewMeta', 'MethodBasedView', )

http_methods = frozenset(['get', 'post', 'head', 'options', 'delete', 'put',
                          'trace', 'patch'])


class View(object):
    """
    Subclass for implementing method-based views.
    """
    @classmethod
    def as_view(cls, name, *class_args, **class_kwargs):
        """Converts the class into an actual view function that can be used
        with the routing system.
        """
        pass


class MethodViewMeta(type):
    """
    Metaclass, which helps to define list of supported methods in
    class-based views.
    """
    def __new__(cls, name, bases, attrs):
        obj = type.__new__(cls, name, bases, attrs)
        # If not defined 'method' attribute, then make and append him to
        # our class-based view
        if 'methods' not in attrs:
            methods = set(obj.methods if hasattr(obj, 'methods') else [])
            for key in attrs:
                if key in http_methods:
                    methods.add(key.lower())
            # This action necessary for appending list of supported methods
            obj.methods = sorted(methods)
        return obj


class MethodBasedView(View, metaclass=MethodViewMeta):
    """
    Method-based view for aiorest-ws framework.
    """
    renderers = ()

    def dispatch(self, request, *args, **kwargs):
        """
        Search the most suitable handler for request.

        :param request: passed request from user.
        """
        method = request.method

        # Invoked, when user not specified method in query (e.c. get, post)
        if not method:
            raise NotSpecifiedMethodName()

        # Invoked, when user specified method name as not a string
        if not isinstance(method, str):
            raise IncorrectMethodNameType()

        # Trying to find the most suitable handler. For that what we are doing:
        # 1) Make string in lowercase (e.c. 'GET' -> 'get')
        # 2) Look into class and try to get handler with this name
        # 3) If extracting is successful, then invoke handler with arguments
        method = method.lower().strip()
        handler = getattr(self, method, None)
        if not handler:
            raise NotSpecifiedHandler()
        return handler(request, *args, **kwargs)

    def get_renderer(self, preferred_format, *args, **kwargs):
        """
        Get serialize class, which using to converting response to
        some users format.

        :param preferred_format: string, which means serializing response to
                                 required format (e.c. json, xml).
        """
        if self.renderers:
            if type(self.renderers) not in (list, tuple):
                raise InvalidRenderer()

            # By default we are take first serializer from list/tuple
            renderer = self.renderers[0]

            if preferred_format:
                # Try to find suitable serializer
                for renderer_class in self.renderers:
                    if renderer_class.format == preferred_format:
                        renderer = renderer_class
                        break

            return renderer()
        else:
            return JSONRenderer()



# -*- coding: utf-8 -*-
"""
Django-like settings functions and classes, which help to modify basic
configuration on the fly, while tests performed.
"""
from functools import wraps
from unittest import TestCase

from aiorest_ws.conf import settings, UserSettingsHolder

__all__ = ('override_settings', )


class BaseContextDecorator(object):
    """
    A base class that can either be used as a context manager during tests
    or as a test function or unittest.TestCase subclass decorator to perform
    temporary alterations.
    `attr_name`: attribute assigned the return value of enable() if used as
                 a class decorator.
    `kwarg_name`: keyword argument passing the return value of enable() if
                  used as a function decorator.
    """
    def __init__(self, attr_name=None, kwarg_name=None):
        self.attr_name = attr_name
        self.kwarg_name = kwarg_name

    def enable(self):
        raise NotImplementedError

    def disable(self):
        raise NotImplementedError

    def __enter__(self):
        return self.enable()

    def __exit__(self, exc_type, exc_value, traceback):
        self.disable()

    def decorate_class(self, cls):
        if issubclass(cls, TestCase):
            decorated_setUp = cls.setUp
            decorated_tearDown = cls.tearDown

            def setUp(inner_self):
                context = self.enable()
                if self.attr_name:
                    setattr(inner_self, self.attr_name, context)
                decorated_setUp(inner_self)

            def tearDown(inner_self):
                decorated_tearDown(inner_self)
                self.disable()

            cls.setUp = setUp
            cls.tearDown = tearDown
            return cls
        raise TypeError('Can only decorate subclasses of unittest.TestCase')

    def decorate_callable(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            with self as context:
                if self.kwarg_name:
                    kwargs[self.kwarg_name] = context
                return func(*args, **kwargs)
        return inner

    def __call__(self, decorated):
        if isinstance(decorated, type):
            return self.decorate_class(decorated)
        elif callable(decorated):
            return self.decorate_callable(decorated)
        raise TypeError('Cannot decorate object of type %s' % type(decorated))


class override_settings(BaseContextDecorator):
    """
    Acts as either a decorator or a context manager. If it's a decorator it
    takes a function and returns a wrapped function. If it's a contextmanager
    it's used with the ``with`` statement. In either event entering/exiting
    are called before and after, respectively, the function/block is executed.
    """
    def __init__(self, **kwargs):
        self.options = kwargs
        super(override_settings, self).__init__()

    def enable(self):
        old_keys = set(key for key in settings.__dir__() if key.isupper())
        new_keys = set(self.options.keys())
        self._new_setting_keys = list(new_keys.difference(old_keys))

        override = UserSettingsHolder(settings._wrapped)
        for key, new_value in self.options.items():
            setattr(override, key, new_value)
        self.wrapped = settings._wrapped
        settings._wrapped = override
        for key, new_value in self.options.items():
            setattr(settings, key, new_value)

    def disable(self):
        settings._wrapped = self.wrapped
        del self.wrapped
        # Rollback old values
        for key in self.options:
            new_value = getattr(settings, key, None)
            setattr(settings, key, new_value)
        # Other keys just remove from the settings
        for key in self._new_setting_keys:
            delattr(settings, key)

    def save_options(self, test_func):
        if test_func._overridden_settings is None:
            test_func._overridden_settings = self.options
        else:
            # Duplicate dict to prevent subclasses from altering their parent
            test_func._overridden_settings = dict(
                test_func._overridden_settings, **self.options
            )

    def decorate_class(self, cls):
        self.save_options(cls)
        return cls



# -*- coding: utf-8 -*-
"""
Middlewares for authentication issues.
"""
from aiorest_ws.abstract import AbstractMiddleware
from aiorest_ws.auth.user.models import UserSQLiteModel
from aiorest_ws.auth.user.abstractions import User
from aiorest_ws.auth.token.backends import InMemoryTokenBackend
from aiorest_ws.auth.token.exceptions import TokenNotProvidedException
from aiorest_ws.auth.token.managers import JSONWebTokenManager
from aiorest_ws.utils.modify import add_property

__all__ = ('BaseTokenMiddleware', 'JSONWebTokenMiddleware', )


class BaseTokenMiddleware(AbstractMiddleware):
    """
    Base token middleware class.
    """

    def init_credentials(self, request):
        """
        Getting credentials (user, keys, tokens) from database/cache/etc.

        :param request: instance of Request class.
        """
        pass

    def authenticate(self, request, handler):
        """
        Authenticate user.

        :param request: instance of Request class.
        :param handler: view, invoked later for the request.
        """
        pass

    def process_request(self, request, handler):
        """
        Processing request before calling handler.

        :param request: instance of Request class.
        :param handler: view, invoked later for the request.
        """
        self.init_credentials(request)
        self.authenticate(request, handler)


class JSONWebTokenMiddleware(BaseTokenMiddleware):
    """
    The JSON Web Token middleware class.
    """
    storage_backend = InMemoryTokenBackend
    manager = JSONWebTokenManager
    user_model = UserSQLiteModel

    def __init__(self):
        super(JSONWebTokenMiddleware, self).__init__()
        self.storage_backend = self.storage_backend()
        self.user_model = self.user_model()
        self.manager = self.manager()

    def get_user_by_token(self, token):
        """
        Get user from the database by passed token.

        :param token: token as string.
        """
        token_data = self.storage_backend.get(token)
        if token_data:
            user = self.user_model.get_user_by_token(token_data)
        else:
            user = User()
        return user

    def init_credentials(self, request):
        """
        Getting credentials (user, keys, tokens) from database/cache/etc.

        :param request: instance of Request class.
        """
        token = getattr(request, 'token', None)

        if token:
            token_payload = self.manager.verify(token)
            user = self.get_user_by_token(token)
        else:
            token_payload = None
            user = User()

        add_property(request, 'user', user)
        add_property(request, 'token_payload', token_payload)

    def authenticate(self, request, view):
        """
        Authenticate user.

        NOTE: Authentication applied for the views, which set `auth_required`
        attribute to `True` value.

        :param request: instance of Request class.
        :param view: view, invoked later for the request.
        """
        auth_required = getattr(view, 'auth_required', False)
        permission_classes = getattr(view, 'permission_classes', ())

        if auth_required:
            if hasattr(request, 'token') and request.token:
                for permission in permission_classes:
                    permission.check(request, view)
            else:
                raise TokenNotProvidedException()



# -*- coding: utf-8 -*-
"""
Helper functions for mapping model fields to a dictionary of default
keyword arguments that should be used for their equivalent serializer fields.
"""
from django.core.validators import DecimalValidator, MinLengthValidator, \
    MaxLengthValidator, MinValueValidator, MaxValueValidator, URLValidator, \
    validate_email, validate_slug, validate_ipv46_address
from django.db import models
from django.utils.text import capfirst


from aiorest_ws.db.orm.django.validators import UniqueValidator
from aiorest_ws.utils.field_mapping import needs_label

__all__ = [
    'NUMERIC_FIELD_TYPES', 'get_detail_view_name', 'get_field_kwargs',
    'get_relation_kwargs', 'get_nested_relation_kwargs', 'get_url_kwargs'
]


NUMERIC_FIELD_TYPES = (
    models.IntegerField, models.FloatField, models.DecimalField
)


def get_detail_view_name(model):
    """
    Given a model class, return the view name to use for URL relationships
    that refer to instances of the model.
    """
    return '%(model_name)s-detail' % {
        'app_label': model._meta.app_label,
        'model_name': model._meta.object_name.lower()
    }


def get_field_kwargs(field_name, model_field):
    """
    Creates a default instance of a basic non-relational field.
    """
    kwargs = {}
    validator_kwarg = list(model_field.validators)

    # The following will only be used by ModelField classes.
    # Gets removed for everything else
    kwargs['model_field'] = model_field

    if model_field.verbose_name and needs_label(model_field.verbose_name, field_name):  # NOQA
        kwargs['label'] = capfirst(model_field.verbose_name)

    if model_field.help_text:
        kwargs['help_text'] = model_field.help_text

    max_digits = getattr(model_field, 'max_digits', None)
    if max_digits is not None:
        kwargs['max_digits'] = max_digits

    decimal_places = getattr(model_field, 'decimal_places', None)
    if decimal_places is not None:
        kwargs['decimal_places'] = decimal_places

    if isinstance(model_field, models.AutoField) or not model_field.editable:
        # If this field is read-only, then return early.
        # Further keyword arguments are not valid
        kwargs['read_only'] = True
        return kwargs

    if model_field.has_default() or model_field.blank or model_field.null:
        kwargs['required'] = False

    is_nullable_field = not isinstance(model_field, models.NullBooleanField)
    if model_field.null and is_nullable_field:
        kwargs['allow_null'] = True

    if model_field.blank and (isinstance(model_field, models.CharField) or
                              isinstance(model_field, models.TextField)):
        kwargs['allow_blank'] = True

    if isinstance(model_field, models.FilePathField):
        kwargs['path'] = model_field.path

        if model_field.match is not None:
            kwargs['match'] = model_field.match

        if model_field.recursive is not False:
            kwargs['recursive'] = model_field.recursive

        if model_field.allow_files is not True:
            kwargs['allow_files'] = model_field.allow_files

        if model_field.allow_folders is not False:
            kwargs['allow_folders'] = model_field.allow_folders

    if model_field.choices:
        # If this model field contains choices, then return early.
        # Further keyword arguments are not valid
        kwargs['choices'] = model_field.choices
        return kwargs

    # Our decimal validation is handled in the field code, not validator code.
    if isinstance(model_field, models.DecimalField):
        validator_kwarg = [
            validator for validator in validator_kwarg
            if not isinstance(validator, DecimalValidator)
        ]

    # Ensure that max_length is passed explicitly as a keyword arg,
    # rather than as a validator
    max_length = getattr(model_field, 'max_length', None)
    if max_length is not None and (isinstance(model_field, models.CharField) or
                                   isinstance(model_field, models.TextField)):
        kwargs['max_length'] = max_length
        validator_kwarg = [
            validator for validator in validator_kwarg
            if not isinstance(validator, MaxLengthValidator)
        ]

    # Ensure that min_length is passed explicitly as a keyword arg,
    # rather than as a validator
    min_length = next((
        validator.limit_value for validator in validator_kwarg
        if isinstance(validator, MinLengthValidator)
    ), None)
    if min_length is not None and isinstance(model_field, models.CharField):
        kwargs['min_length'] = min_length
        validator_kwarg = [
            validator for validator in validator_kwarg
            if not isinstance(validator, MinLengthValidator)
        ]

    # Ensure that max_value is passed explicitly as a keyword arg,
    # rather than as a validator
    max_value = next((
        validator.limit_value for validator in validator_kwarg
        if isinstance(validator, MaxValueValidator)
    ), None)
    if max_value is not None and isinstance(model_field, NUMERIC_FIELD_TYPES):
        kwargs['max_value'] = max_value
        validator_kwarg = [
            validator for validator in validator_kwarg
            if not isinstance(validator, MaxValueValidator)
        ]

    # Ensure that max_value is passed explicitly as a keyword arg,
    # rather than as a validator
    min_value = next((
        validator.limit_value for validator in validator_kwarg
        if isinstance(validator, MinValueValidator)
    ), None)
    if min_value is not None and isinstance(model_field, NUMERIC_FIELD_TYPES):
        kwargs['min_value'] = min_value
        validator_kwarg = [
            validator for validator in validator_kwarg
            if not isinstance(validator, MinValueValidator)
        ]

    # URLField does not need to include the URLValidator argument,
    # as it is explicitly added in
    if isinstance(model_field, models.URLField):
        validator_kwarg = [
            validator for validator in validator_kwarg
            if not isinstance(validator, URLValidator)
        ]

    # EmailField does not need to include the validate_email argument,
    # as it is explicitly added in
    if isinstance(model_field, models.EmailField):
        validator_kwarg = [
            validator for validator in validator_kwarg
            if validator is not validate_email
        ]

    # SlugField do not need to include the 'validate_slug' argument
    if isinstance(model_field, models.SlugField):
        validator_kwarg = [
            validator for validator in validator_kwarg
            if validator is not validate_slug
        ]

    # for IPAddressField exclude the 'validate_ipv46_address' argument
    if isinstance(model_field, models.GenericIPAddressField):
        validator_kwarg = [
            validator for validator in validator_kwarg
            if validator is not validate_ipv46_address
        ]

    if getattr(model_field, 'unique', False):
        unique_error_message = model_field.error_messages.get('unique', None)
        if unique_error_message:
            unique_error_message = unique_error_message % {
                'model_name': model_field.model._meta.verbose_name,
                'field_label': model_field.verbose_name
            }
        validator = UniqueValidator(
            queryset=model_field.model._default_manager,
            message=unique_error_message
        )
        validator_kwarg.append(validator)

    if validator_kwarg:
        kwargs['validators'] = validator_kwarg

    return kwargs


def get_relation_kwargs(field_name, relation_info):
    """
    Creates a default instance of a flat relational field.
    """
    model_field = relation_info.model_field
    related_model = relation_info.related_model
    to_many = relation_info.to_many
    to_field = relation_info.to_field
    has_through_model = relation_info.has_through_model

    kwargs = {
        'queryset': related_model._default_manager,
        'view_name': get_detail_view_name(related_model)
    }

    if to_many:
        kwargs['many'] = True

    if to_field:
        kwargs['to_field'] = to_field

    if has_through_model:
        kwargs['read_only'] = True
        kwargs.pop('queryset', None)

    if model_field:
        if model_field.verbose_name and needs_label(model_field, field_name):
            kwargs['label'] = capfirst(model_field.verbose_name)
        if model_field.help_text:
            kwargs['help_text'] = model_field.help_text
        if not model_field.editable:
            kwargs['read_only'] = True
            kwargs.pop('queryset', None)
        if kwargs.get('read_only', False):
            # If this field is read-only, then return early.
            # No further keyword arguments are valid
            return kwargs

        if model_field.has_default() or model_field.blank or model_field.null:
            kwargs['required'] = False
        if model_field.null:
            kwargs['allow_null'] = True
        if model_field.validators:
            kwargs['validators'] = model_field.validators
        if getattr(model_field, 'unique', False):
            queryset = model_field.model._default_manager
            validator = UniqueValidator(queryset=queryset)
            kwargs['validators'] = kwargs.get('validators', []) + [validator]
        if to_many and not model_field.blank:
            kwargs['allow_empty'] = False

    return kwargs


def get_nested_relation_kwargs(relation_info):
    kwargs = {'read_only': True}
    if relation_info.to_many:
        kwargs['many'] = True
    return kwargs


def get_url_kwargs(model_class):
    return {
        'view_name': get_detail_view_name(model_class)
    }



# -*- coding: utf-8 -*-
"""
Module which provide classes and function for related and nested field.

NOTE: Don't forget to override `to_internal_value()`, `to_representation()`
and other specific methods for all specified classes for you own purposes.

For your own ORM support necessary to implement classes accordingly to the
next inheritance tree (don't look onto the classes, which aren't inherited
as on the picture):

    AbstractField
    ├─ ManyRelatedField
    └─ RelatedField
       ├─ StringRelatedField
       ├─ PrimaryKeyRelatedField
       ├─ HyperlinkedRelatedField
       │  └─ HyperlinkedIdentityField
       └─ SlugRelatedField  # Inherit from it, if your ORM provide slug field
"""
import collections
from urllib.parse import urlparse

from aiorest_ws.conf import settings
from aiorest_ws.db.orm.abstract import AbstractField, empty
from aiorest_ws.exceptions import ImproperlyConfigured
from aiorest_ws.urls.exceptions import NoReverseMatch, NoMatch
from aiorest_ws.urls.utils import reverse, resolve
from aiorest_ws.utils.fields import method_overridden

__all__ = (
    'MANY_RELATION_KWARGS', 'Hyperlink', 'PKOnlyObject', 'RelatedField',
    'ManyRelatedField', 'StringRelatedField', 'PrimaryKeyRelatedField',
    'HyperlinkedRelatedField', 'HyperlinkedIdentityField', 'SlugRelatedField'
)

# We assume that 'validators' are intended for the child serializer,
# rather than the parent serializer
MANY_RELATION_KWARGS = (
    'read_only', 'write_only', 'required', 'default', 'initial', 'source',
    'label', 'help_text', 'style', 'error_messages', 'allow_empty'
)


class Hyperlink(str):
    """
    A string like object that additionally has an associated name.
    We use this for hyperlinked URLs that may render as a named link
    in some contexts, or render as a plain URL in others.
    """
    is_hyperlink = True

    def __new__(self, url, name):
        ret = str.__new__(self, url)
        ret.name = name
        return ret

    def __getnewargs__(self):
        return str(self), self.name


class PKOnlyObject(object):
    """
    This is a mock object, used for when we only need the pk of the object
    instance, but still want to return an object with a .pk attribute,
    in order to keep the same interface as a regular model instance.
    """
    def __init__(self, pk):
        self.pk = pk


class RelatedField(AbstractField):
    queryset = None
    many_related_field = None

    def __init__(self, **kwargs):
        self.queryset = kwargs.pop('queryset', self.queryset)

        field_is_read_only = kwargs.get('read_only', None)
        if not method_overridden('get_queryset', RelatedField, self):
            assert self.queryset is not None or field_is_read_only, (
                'Relational field must provide a `queryset` argument, '
                'override `get_queryset`, or set read_only=`True`.'
            )
        assert not (self.queryset is not None and field_is_read_only), (
            'Relational fields should not provide a `queryset` argument, '
            'when setting read_only=`True`.'
        )
        kwargs.pop('many', None)
        kwargs.pop('allow_empty', None)
        super(RelatedField, self).__init__(**kwargs)

    def __new__(cls, *args, **kwargs):
        # We override this method in order to automagically create
        # `ManyRelatedField` classes instead when `many=True` is set
        if kwargs.pop('many', False):
            return cls.many_init(*args, **kwargs)
        return super(RelatedField, cls).__new__(cls, *args, **kwargs)

    @classmethod
    def many_init(cls, *args, **kwargs):
        """
        This method handles creating a parent `ManyRelatedField` instance
        when the `many=True` keyword argument is passed.
        Typically you won't need to override this method.
        Note that we're over-cautious in passing most arguments to both parent
        and child classes in order to try to cover the general case. If you're
        overriding this method you'll probably want something much simpler, eg:

        @classmethod
        def many_init(cls, *args, **kwargs):
            kwargs['child'] = cls()
            return CustomManyRelatedField(*args, **kwargs)
        """
        list_kwargs = {'child_relation': cls(*args, **kwargs)}
        for key in kwargs.keys():
            if key in MANY_RELATION_KWARGS:
                list_kwargs[key] = kwargs[key]
        return cls.many_related_field(**list_kwargs)

    def run_validation(self, data=empty):
        # We force empty strings to None values for relational fields.
        if data == '':
            data = None
        return super(RelatedField, self).run_validation(data)

    def get_queryset(self):
        raise NotImplementedError('`get_queryset()` must be implemented.')

    def use_pk_only_optimization(self):
        return False

    def get_attribute(self, instance):
        raise NotImplementedError('`get_attribute()` must be implemented.')

    @property
    def choices(self):
        queryset = self.get_queryset()
        if queryset is None:
            # Ensure that field.choices returns something sensible
            # even when accessed with a read-only field.
            return {}

        return collections.OrderedDict([
            (str(self.to_representation(item)), self.display_value(item))
            for item in queryset
        ])

    @property
    def grouped_choices(self):
        return self.choices

    def display_value(self, instance):
        return str(instance)


class ManyRelatedField(AbstractField):
    """
    Relationships with `many=True` transparently get coerced into instead being
    a ManyRelatedField with a child relationship.
    The `ManyRelatedField` class is responsible for handling iterating through
    the values and passing each one to the child relationship.
    This class is treated as private API.
    You shouldn't generally need to be using this class directly yourself,
    and should instead simply set 'many=True' on the relationship.
    """
    initial = []
    default_error_messages = {
        'not_a_list': u'Expected a list of items but got type "{input_type}".',
        'empty': u'This list may not be empty.'
    }

    def __init__(self, child_relation=None, *args, **kwargs):
        self.child_relation = child_relation
        self.allow_empty = kwargs.pop('allow_empty', True)
        assert child_relation is not None, '`child_relation` is a required ' \
                                           'argument.'
        super(ManyRelatedField, self).__init__(*args, **kwargs)
        self.child_relation.bind(field_name='', parent=self)

    @property
    def choices(self):
        return self.child_relation.choices

    @property
    def grouped_choices(self):
        return self.choices

    def get_value(self, dictionary):
        return dictionary.get(self.field_name, empty)

    def get_attribute(self, instance):
        raise NotImplementedError('`get_attribute()` must be implemented.')

    def to_internal_value(self, data):
        if isinstance(data, type('')) or not hasattr(data, '__iter__'):
            self.raise_error('not_a_list', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            self.raise_error('empty')

        return [
            self.child_relation.to_internal_value(item)
            for item in data
        ]

    def to_representation(self, iterable):
        return [
            self.child_relation.to_representation(value)
            for value in iterable
        ]


class StringRelatedField(object):
    """
    A read only field that represents its targets using their plain
    string representation.
    """
    def __init__(self, **kwargs):
        kwargs['read_only'] = True
        super(StringRelatedField, self).__init__(**kwargs)

    def to_representation(self, value):
        return str(value)


class PrimaryKeyRelatedField(object):
    default_error_messages = {
        'required': u'This field is required.',
        'does_not_exist': u'Invalid pk "{pk_value}" - object does not exist.',
        'incorrect_type': u'Incorrect type. Expected pk value, received '
                          u'{data_type}.',
    }

    def __init__(self, **kwargs):
        self.pk_field = kwargs.pop('pk_field', None)
        super(PrimaryKeyRelatedField, self).__init__(**kwargs)

    def use_pk_only_optimization(self):
        return True


class HyperlinkedRelatedField(object):
    lookup_field = None
    view_name = None

    default_error_messages = {
        'required': u'This field is required.',
        'no_match': u'Invalid hyperlink - No URL match.',
        'no_primary_key': u'For every instance necessary to specify '
                          u'`{url_field}` field.',
        'does_not_exist': u'Invalid hyperlink - Object does not exist.',
        'incorrect_type': u'Incorrect type. Expected URL string, '
                          u'received {data_type}.',
    }

    def __init__(self, view_name=None, **kwargs):
        if view_name is not None:
            self.view_name = view_name
        assert self.view_name is not None, 'The `view_name` argument ' \
                                           'is required.'
        self.lookup_field = kwargs.pop('lookup_field', self.lookup_field)
        self.lookup_url_kwarg = kwargs.pop(
            'lookup_url_kwarg', self.lookup_field
        )
        self.format = kwargs.pop('format', None)

        # We include this simply for dependency injection in tests.
        # We can't add it as a class attributes or it would expect an
        # implicit `self` argument to be passed.
        # Set the `reverse` attribute of this class
        self.reverse = reverse

        super(HyperlinkedRelatedField, self).__init__(**kwargs)

    def use_pk_only_optimization(self):
        # Must return boolean value for equal operation between
        # self.lookup_field attribute and defined model PK.
        # For example, for Django Framework it can be:
        # return self.lookup_field == 'pk'
        raise NotImplementedError("`use_pk_only_optimization()` must be "
                                  "implemented.")

    def get_object(self, view_name, view_args, view_kwargs):
        """
        Return the object corresponding to a matched URL.
        Takes the matched URL conf arguments, and should return an
        object instance, or raise exception for a not existing object.
        """
        raise NotImplementedError('`get_object()` must be implemented.')

    def is_saved_in_database(self, obj):
        """
        Return the boolean value, which let us to understand whether saved
        object in a database or not.
        """
        raise NotImplementedError('`is_saved_in_database()` must be '
                                  'implemented.')

    def get_lookup_value(self, obj):
        """
        Return a tuple of object lookup values, which are will be using for
        reverse operation.
        """
        raise NotImplementedError('`get_lookup_value()` must be implemented.')

    def get_url(self, obj, view_name):
        """
        Given an object, return the URL that hyperlinks to the object.
        May raise a `NoReverseMatch` if the `view_name` and `lookup_field`
        attributes are not configured to correctly match the URL conf.
        """
        # Unsaved objects will not yet have a valid URL
        if not self.is_saved_in_database(obj):
            return None

        # If have taken `context` and set `relative=True`, then necessary to
        # generate relative URL
        relative = self.context.get('relative', False)

        args = tuple(map(str, self.get_lookup_value(obj)))
        return self.reverse(view_name, args=args, relative=relative)

    def get_name(self, obj):
        return str(obj)

    def to_internal_value(self, data):
        url_field_name = settings.REST_CONFIG['URL_FIELD_NAME']

        try:
            url = urlparse(data[url_field_name]).path
        except KeyError:
            self.raise_error('no_primary_key', url_field=url_field_name)
        except (AttributeError, TypeError):
            self.raise_error('incorrect_type', data_type=type(data).__name__)

        if not url.startswith('/'):
            url = '/' + url
        if not url.endswith('/'):
            url += '/'

        try:
            match = resolve(url)
        except NoMatch:
            self.raise_error('no_match')

        return self.get_object(match.view_name, match.args, match.kwargs)

    def to_representation(self, value):
        # Return the hyperlink, or error if incorrectly configured
        try:
            url = self.get_url(value, self.view_name)
        except NoReverseMatch:
            msg = (
                'Could not resolve URL for hyperlinked relationship using '
                'view name "%s". You may have failed to include the related '
                'model in your API, or incorrectly configured the '
                '`lookup_field` attribute on this field.'
            )
            if value in ('', None):
                value_string = {'': 'the empty string', None: 'None'}[value]
                msg += (
                    " WARNING: The value of the field on the model instance "
                    "was %s, which may be why it didn't match any "
                    "entries in your URL conf." % value_string
                )
            raise ImproperlyConfigured(msg % self.view_name)

        if url is None:
            return None

        name = self.get_name(value)
        return Hyperlink(url, name)


class HyperlinkedIdentityField(object):
    """
    A read-only field that represents the identity URL for an object, itself.
    This is in contrast to `HyperlinkedRelatedField` which represents the
    URL of relationships to other objects.
    """
    def __init__(self, view_name=None, **kwargs):
        assert view_name is not None, 'The `view_name` argument is required.'
        kwargs['read_only'] = True
        kwargs['source'] = '*'
        super(HyperlinkedIdentityField, self).__init__(view_name, **kwargs)

    def use_pk_only_optimization(self):
        # We have the complete object instance already. We don't need
        # to run the 'only get the pk for this relationship' code
        return False


class SlugRelatedField(object):
    """
    A read-write field that represents the target of the relationship
    by a unique 'slug' attribute.
    """
    default_error_messages = {
        'does_not_exist': u'Object with {slug_name}={value} does not exist.',
        'invalid': u'Invalid value.',
    }

    def __init__(self, slug_field=None, **kwargs):
        assert slug_field is not None, 'The `slug_field` argument is required.'
        self.slug_field = slug_field
        super(SlugRelatedField, self).__init__(**kwargs)



# -*- coding: utf-8 -*-
"""
This module provide structures for data representation of received model.
"""
from collections import namedtuple

__all__ = ('FieldInfo', 'RelationInfo')


FieldInfo = namedtuple('FieldResult', [
    'pk',  # Model field instance
    'fields',  # Dict of field name -> model field instance
    'forward_relations',  # Dict of field name -> RelationInfo
    'reverse_relations',  # Dict of field name -> RelationInfo
    'fields_and_pk',  # Shortcut for 'pk' + 'fields'
    'relations'  # Shortcut for 'forward_relations' + 'reverse_relations'
])

RelationInfo = namedtuple('RelationInfo', [
    'model_field',
    'related_model',
    'to_many',
    'to_field',
    'has_through_model'
])



# -*- coding: utf-8 -*-
"""
Utility module for work with _urlconf variable.
"""
import re

from aiorest_ws.parsers import DYNAMIC_PARAMETER
from aiorest_ws.urls.base import get_urlconf
from aiorest_ws.urls.exceptions import NoReverseMatch, NoMatch
from aiorest_ws.utils.encoding import force_text

__all__ = (
    'RouteMatch', '_generate_url_parameters', 'reverse', 'resolve'
)


class RouteMatch(object):

    def __init__(self, view_name, args=(), kwargs={}):
        super(RouteMatch, self).__init__()
        self.view_name = view_name
        self.args = args
        self.kwargs = kwargs


def _generate_url_parameters(parameters):
    format_parameters = (
        "{}={}".format(key, value)
        for key, value in parameters.items()
    )
    return '?' + '&'.join(format_parameters)


def resolve(path, urlconf=None):
    """
    Return the endpoint corresponding to a matched URL.
    """
    if urlconf is None:
        urlconf = get_urlconf()

    # Convert absolute path to relative
    path = path.replace(urlconf['path'], '')

    # Iterate over the all endpoints
    for route in urlconf.get('urls', []):
        # Find match between endpoint and passed URL
        match = route.match(path)
        if match is not None:
            kwargs = {}
            if route._pattern:
                kwargs = route._pattern.match(path).groupdict()
            return RouteMatch(route.name, args=match, kwargs=kwargs)
    raise NoMatch()


def reverse(view_name, urlconf=None, args=[], kwargs={}, relative=False):
    """
    Generate URL to endpoint, which processing by Application instance, based
    on the `view_name` and passed arguments.

    :param view_name: view name.
    :param urlconf: urlconf instance (dictionary).
    :param args: tuple of data, which used by handler with this URL.
    :param kwargs: named arguments for the defined URL.
    :return: generated URL with the passed arguments.
    """
    if urlconf is None:
        urlconf = get_urlconf()

    root_path = ''
    api_path = ''
    try:
        # Get root path, when necessary to generate absolute URL
        root_path = '' if relative else urlconf['path'].strip('/')

        # Get path to a specific endpoint
        route = urlconf['routes'][view_name]
        api_path = route.path.strip('/')

        # Replace parameters in 'api_url' on the passed args
        dynamic_parameters = re.findall(DYNAMIC_PARAMETER, api_path)
        if len(dynamic_parameters) != len(args):
            raise ValueError(
                "Endpoint '{path}' must take {valid_count} parameters, "
                "but passed {invalid_count}.".format(
                    path=api_path,
                    valid_count=len(dynamic_parameters),
                    invalid_count=len(args)
                )
            )
        for parameter, value in zip(dynamic_parameters, args):
            api_path = api_path.replace(parameter, value)
    except KeyError:
        raise NoReverseMatch()

    url_parameters = _generate_url_parameters(kwargs) if kwargs else ""
    url = '/'.join([root_path, api_path, url_parameters])
    return force_text(url)



# -*- coding: utf-8 -*-
"""
Functions for processing data from raw SQL queries.
"""
__all__ = ('convert_db_row_to_dict', )


def convert_db_row_to_dict(row, mapped_fields):
    """
    Convert row of database to dictionary.

    :param row: row of database.
    :param mapped_fields: list of tuple, which means field names.
    """
    data = {}
    for field, value in zip(mapped_fields, row):
        data[field] = value
    return data



# -*- coding: utf-8 -*-
"""
Special classes and function which help to work with strings correctly.
"""
from aiorest_ws.utils.encoding import force_text

__all__ = ('capfirst', )


def capfirst(x):
    return x and force_text(x)[0].upper() + force_text(x)[1:]



# -*- coding: utf-8 -*-
"""
Base exception classes for serialize and validation actions.
"""
from aiorest_ws.exceptions import BaseAPIException, SerializerError
from aiorest_ws.utils.encoding import force_text_recursive

__all__ = ('ModelSerializerError', 'ValidationError', )


class ModelSerializerError(SerializerError):
    default_detail = u"Error has occurred inside model serializer class."


class ValidationError(BaseAPIException):
    default_detail = u"Validation error has occurred at validation process."

    def __init__(self, detail):
        if not isinstance(detail, dict) and not isinstance(detail, list):
            detail = [detail, ]
        self.detail = force_text_recursive(detail)

    def __str__(self):
        return str(self.detail)



# -*- coding: utf-8 -*-
"""
Permission classes for authentication.
"""
from aiorest_ws.abstract import AbstractPermission

__all__ = ('IsAuthenticated', )


class IsAuthenticated(AbstractPermission):
    """
    Permissions used for checking authenticated users.
    """
    @staticmethod
    def check(request, handler):
        """
        Check permission method.

        :param request: instance of Request class.
        :param handler: view, invoked later for the request.
        """
        return request.user and request.user.is_authenticated()



# -*- coding: utf-8 -*-
"""
This modules provide a functions and classes, which every developer can use
for determine URL for their APIs.

For example, we can use this features something like this:

    router = SimpleRouter()
    router.register('user/info', info_handler, methods='GET')
    router.register('user/register', register_handler, methods='POST')
    router.register('user/profile/{user_name}', user_handler,
                    methods=['GET', 'PUT'])
"""
from aiorest_ws.abstract import AbstractEndpoint, AbstractRouter
from aiorest_ws.exceptions import BaseAPIException, EndpointValueError, \
    NotSpecifiedHandler, NotSpecifiedURL
from aiorest_ws.log import logger
from aiorest_ws.renderers import JSONRenderer
from aiorest_ws.parsers import URLParser
from aiorest_ws.validators import RouteArgumentsValidator
from aiorest_ws.wrappers import Response

__all__ = ('SimpleRouter', )


class SimpleRouter(AbstractRouter):
    """
    Default router class, used for working with REST over WebSockets.
    """
    args_validator = RouteArgumentsValidator()
    url_parser = URLParser()

    def _correct_path(self, path):
        """
        Convert path to valid value.

        :param path: URL, which used to get access to API.
        """
        path = path.strip()
        if not path.endswith('/'):
            path += '/'
        return path

    def register(self, path, handler, methods, name=None):
        """
        Add new endpoint to the router.

        :param path: URL, which used to get access to API.
        :param handler: inherited class from the MethodBasedView, which used
                        for processing request.
        :param methods: list of available for user methods or string with
                        concrete method name.
        :param name: short name for endpoint.
        """
        path = self._correct_path(path)
        self.args_validator.validate(path, handler, methods, name)

        route = self.url_parser.define_route(path, handler, methods, name)
        self._register_url(route)

    def register_endpoint(self, endpoint):
        """
        Add new endpoint to the router.

        :param endpoint: function with @endpoint decorator, which used for
                         processing request.
        """
        path, handler, methods, name = endpoint()
        self.register(path, handler, methods, name)

    def extract_url(self, request):
        """
        Extracting URL parameter for request.

        :param request: request from the user.
        """
        if not request.url:
            raise NotSpecifiedURL()
        return self._correct_path(request.url)

    def search_handler(self, request, url):
        """
        Searching handler by URL.

        :param request: request from user.
        :param url: path to the registered endpoint.
        """
        args = ()
        kwargs = {}
        handler = None
        # Iterate over the all endpoints
        for route in self._urls:
            # Find match between endpoint and request URL
            match = route.match(url)
            if match is not None:
                handler = route.handler()
                args = match
                parameters = request.args
                if parameters:
                    kwargs.update(parameters)
                break
        return handler, args, kwargs

    def process_request(self, request):
        """
        Handle received request from user.

        :param request: request from user.
        """
        logger.info("\"{method} {url}\" args={args}".format(
            method=request.method,
            url=request.url,
            args=request.args)
        )
        response = Response()

        try:
            url = self.extract_url(request)
            handler, args, kwargs = self.search_handler(request, url)

            # Invoke handler for request
            if handler:

                for middleware in self.middlewares:
                    middleware.process_request(request, handler)

                # Search serializer for response
                format = request.get_argument('format')
                serializer = handler.get_renderer(format, *args, **kwargs)

                response.content = handler.dispatch(request, *args, **kwargs)
            else:
                raise NotSpecifiedHandler()
        except BaseAPIException as exc:
            logger.exception(exc)
            response.wrap_exception(exc)
            serializer = JSONRenderer()

        response.append_request(request)
        return serializer.render(response.content)

    def _register_url(self, route):
        """
        Register new endpoint.

        :param route: instance of class, inherited from AbstractEndpoint.
        """
        if not issubclass(type(route), (AbstractEndpoint, )):
            raise TypeError(u"Custom route must be inherited from the "
                            u"AbstractEndpoint class.")

        if route.name:
            if route.name in self._routes.keys():
                raise EndpointValueError(
                    'Duplicate {}, already handled by {}'
                    .format(route.name, self._routes[route.name])
                )
            self._routes[route.name] = route
        self._urls.append(route)

    def include(self, router):
        """
        Appending endpoints from another router to self.

        :param router: instance of subclass, derived from AbstractRouter.
        """
        if not issubclass(type(router), (AbstractRouter, )):
            raise TypeError(u"Passed router must be inherited from the "
                            u"AbstractRouter class.")
        self._urls.extend(router._urls)
        self._routes.update(router._routes)



# -*- coding: utf-8 -*-
"""
Exceptions for user management.
"""
from aiorest_ws.exceptions import BaseAPIException

__all__ = (
    'RequiredModelFieldsNotDefined', 'SearchCriteriaRequired',
    'NotEnoughArguments',
)


class RequiredModelFieldsNotDefined(BaseAPIException):
    default_detail = u"Required fields aren't defined."


class SearchCriteriaRequired(BaseAPIException):
    default_detail = u"Criteria for WHEN statement not defined."


class NotEnoughArguments(BaseAPIException):
    default_detail = u"Necessary specify at least one field to update."



# -*- coding: utf-8 -*-
"""
Exception classes for work with model serializers, fields and SQLAlchemy ORM.
"""
from aiorest_ws.exceptions import BaseAPIException


__all__ = ('ObjectDoesNotExist', )


class ObjectDoesNotExist(BaseAPIException):
    default_detail = "The requested object does not exist."



# -*- coding: utf-8 -*-
"""
Backend classes for storages.
"""
__all__ = ('BaseStorageBackend', )


class BaseStorageBackend(object):
    """
    Base interface for storage.
    """
    def get(self, *args, **kwargs):
        """
        Get object from the storage.
        """
        pass

    def save(self, *args, **kwargs):
        """
        Save object in the storage.
        """
        pass



# -*- coding: utf-8 -*-
"""
Default implementations of ORM column types.

This fields doesn't used with ORMs by default, like Django or SQLAlchemy ORM.
For every field which are provided by ORM, necessary to implement special
mixin class, which extract value from container.
"""
import datetime
import decimal
import inspect
import json
import collections
import copy
import re

from aiorest_ws.conf import settings
from aiorest_ws.db.orm import validators
from aiorest_ws.db.orm.abstract import AbstractField, SkipField, empty
from aiorest_ws.utils.encoding import force_text
from aiorest_ws.utils.date import humanize_datetime, timezone, dateparse
from aiorest_ws.utils.fields import to_choices_dict, flatten_choices_dict

__all__ = (
    'IntegerField', 'BigIntegerField', 'SmallIntegerField', 'BooleanField',
    'NullBooleanField', 'CharField', 'ChoiceField', 'FloatField',
    'LargeBinaryField', 'DecimalField', 'TimeField', 'DateField',
    'PickleField', 'DateTimeField', 'TimeDeltaField', 'ListField', 'DictField',
    'JSONField', 'HStoreField', 'ModelField', 'HiddenField',
    'CreateOnlyDefault', 'ReadOnlyField', 'SerializerMethodField',
    '_UnvalidatedField'
)


class IntegerField(AbstractField):
    default_error_messages = {
        'invalid': u"Passed value must be an integer.",
        'max_value': u"Ensure this value is less than or equal "
                     u"to {max_value}.",
        'min_value': u"Ensure this value is greater than or equal "
                     u"to {min_value}.",
        'max_string_length': u"String value too large."
    }
    MAX_STRING_LENGTH = 1000  # protect against extremely string inputs
    re_decimal = re.compile(r'\.0*$')  # allow to use .0 at the end of number

    def __init__(self, **kwargs):
        self.max_value = kwargs.pop('max_value', None)
        self.min_value = kwargs.pop('min_value', None)
        super(IntegerField, self).__init__(**kwargs)
        if self.max_value is not None:
            message = self.error_messages['max_value'].format(max_value=self.max_value)  # NOQA
            self.validators.append(validators.MaxValueValidator(self.max_value, message=message))  # NOQA
        if self.min_value is not None:
            message = self.error_messages['min_value'].format(min_value=self.min_value)  # NOQA
            self.validators.append(validators.MinValueValidator(self.min_value, message=message))  # NOQA

    def to_internal_value(self, data):
        if isinstance(data, str) and len(data) > self.MAX_STRING_LENGTH:
            self.raise_error('max_string_length')

        try:
            data = int(self.re_decimal.sub('', str(data)))
        except (ValueError, TypeError):
            self.raise_error('invalid')
        return data

    def to_representation(self, value):
        return int(value)


class BigIntegerField(IntegerField):
    MAX_BIG_INTEGER = 9223372036854775807

    def __init__(self, **kwargs):
        kwargs['min_value'] = -self.MAX_BIG_INTEGER - 1
        kwargs['max_value'] = self.MAX_BIG_INTEGER
        super(BigIntegerField, self).__init__(**kwargs)


class SmallIntegerField(IntegerField):
    MAX_SMALL_INTEGER = 32767

    def __init__(self, **kwargs):
        kwargs['min_value'] = -self.MAX_SMALL_INTEGER - 1
        kwargs['max_value'] = self.MAX_SMALL_INTEGER
        super(SmallIntegerField, self).__init__(**kwargs)


class BooleanField(AbstractField):
    default_error_messages = {
        'invalid': u'"{input}" is not a valid boolean."'
    }
    initial = False
    TRUE_VALUES = {'true', 'True', 'TRUE', '1', 1, True}
    FALSE_VALUES = {'false', 'False', 'FALSE', '0', 0, 0.0, False}

    def __init__(self, **kwargs):
        assert 'allow_null' not in kwargs, \
            '`allow_null` is not a valid option. ' \
            'Use `NullBooleanField` instead.'
        super(BooleanField, self).__init__(**kwargs)

    def to_internal_value(self, data):
        try:
            if data in self.TRUE_VALUES:
                return True
            elif data in self.FALSE_VALUES:
                return False
        except TypeError:  # Input is an unhashable type
            pass
        self.raise_error('invalid', input=data)

    def to_representation(self, value):
        if value in self.TRUE_VALUES:
            return True
        elif value in self.FALSE_VALUES:
            return False
        return bool(value)


class NullBooleanField(AbstractField):
    default_error_messages = {
        'invalid': u'"{input}" is not a valid boolean."'
    }
    initial = None
    TRUE_VALUES = {'true', 'True', 'TRUE', '1', 1, True}
    FALSE_VALUES = {'false', 'False', 'FALSE', '0', 0, 0.0, False}
    NULL_VALUES = {'null', 'Null', 'NULL', '', None}

    def __init__(self, **kwargs):
        kwargs['allow_null'] = True
        super(NullBooleanField, self).__init__(**kwargs)

    def to_internal_value(self, data):
        if data in self.TRUE_VALUES:
            return True
        elif data in self.FALSE_VALUES:
            return False
        elif data in self.NULL_VALUES:
            return None
        self.raise_error('invalid', input=data)

    def to_representation(self, value):
        if value in self.NULL_VALUES:
            return None
        if value in self.TRUE_VALUES:
            return True
        elif value in self.FALSE_VALUES:
            return False
        return bool(value)


class CharField(AbstractField):
    default_error_messages = {
        'blank': u"This field may not be blank.",
        'max_length': "Ensure this field has no more than "
                      "{max_length} characters.",
        'min_length': "Ensure this field has at least "
                      "{min_length} characters."
    }
    initial = ''

    def __init__(self, **kwargs):
        self.allow_blank = kwargs.pop('allow_blank', False)
        self.trim_whitespace = kwargs.pop('trim_whitespace', True)
        self.max_length = kwargs.pop('max_length', None)
        self.min_length = kwargs.pop('min_length', None)
        super(CharField, self).__init__(**kwargs)
        if self.max_length is not None:
            message = self.error_messages['max_length'].format(
                max_length=self.max_length
            )
            self.validators.append(
                validators.MaxLengthValidator(self.max_length, message=message)
            )
        if self.min_length is not None:
            message = self.error_messages['min_length'].format(
                min_length=self.min_length
            )
            self.validators.append(
                validators.MinLengthValidator(self.min_length, message=message)
            )

    def run_validation(self, data=empty):
        # Test for the empty string here so that it does not get validated,
        # and so that subclasses do not need to handle it explicitly inside
        # the `to_internal_value()` method
        if data == '' or (self.trim_whitespace and str(data).strip() == ''):
            if not self.allow_blank:
                self.raise_error('blank')
            return ''
        return super(CharField, self).run_validation(data)

    def to_internal_value(self, data):
        value = str(data)
        return value.strip() if self.trim_whitespace else value

    def to_representation(self, value):
        return str(value)


class ChoiceField(AbstractField):
    default_error_messages = {
        'invalid_choice': '"{input}" is not a valid choice.'
    }

    def __init__(self, choices, **kwargs):
        self.grouped_choices = to_choices_dict(choices)
        self.choices = flatten_choices_dict(self.grouped_choices)

        # Map the string representation of choices to the underlying value.
        # Allows us to deal with eg. integer choices while supporting either
        # integer or string input, but still get the correct datatype out
        self.choice_strings_to_values = {
            str(key): key for key in self.choices.keys()
        }

        self.allow_blank = kwargs.pop('allow_blank', False)
        super(ChoiceField, self).__init__(**kwargs)

    def to_internal_value(self, data):
        if data == '' and self.allow_blank:
            return ''

        try:
            return self.choice_strings_to_values[str(data)]
        except KeyError:
            self.raise_error('invalid_choice', input=data)

    def to_representation(self, value):
        if value in ('', None):
            return value
        return self.choice_strings_to_values.get(str(value), value)


class FloatField(AbstractField):
    default_error_messages = {
        'invalid': u"Passed value must be a float.",
        'max_value': u"Ensure this value is less than or equal "
                     u"to {max_value}.",
        'min_value': u"Ensure this value is greater than or equal "
                     u"to {min_value}.",
        'max_string_length': u"String value too large."
    }
    MAX_STRING_LENGTH = 1000  # protect against extremely string inputs

    def __init__(self, **kwargs):
        self.max_value = kwargs.pop('max_value', None)
        self.min_value = kwargs.pop('min_value', None)
        super(FloatField, self).__init__(**kwargs)
        if self.max_value is not None:
            message = self.error_messages['max_value'].format(
                max_value=self.max_value
            )
            self.validators.append(
                validators.MaxValueValidator(self.max_value, message=message)
            )
        if self.min_value is not None:
            message = self.error_messages['min_value'].format(
                min_value=self.min_value
            )
            self.validators.append(
                validators.MinValueValidator(self.min_value, message=message)
            )

    def to_internal_value(self, data):
        if isinstance(data, str) and len(data) > self.MAX_STRING_LENGTH:
            self.raise_error('max_string_length')

        try:
            return float(data)
        except (TypeError, ValueError):
            self.raise_error('invalid')

    def to_representation(self, value):
        return float(value)


class PickleField(AbstractField):
    default_error_messages = {
        'invalid': u"Passed value must be a bytes.",
    }

    def to_internal_value(self, data):
        if not isinstance(data, bytes):
            self.raise_error('invalid')
        return data

    def to_representation(self, value):
        return bytes(value)


class LargeBinaryField(AbstractField):
    default_error_messages = {
        'max_binary_length': u"Binary value too large."
    }

    def __init__(self, length=None, *args, **kwargs):
        self.length = length
        super(LargeBinaryField, self).__init__(*args, **kwargs)

    def to_internal_value(self, data):
        if self.length and len(data) > self.length:
            self.raise_error('max_binary_length')

        return bytes(data, encoding='utf-8')

    def to_representation(self, value):
        return bytes(value, encoding='utf-8')


class TimeField(AbstractField):
    default_error_messages = {
        'invalid': u"Time has wrong format. Use one of these "
                   u"formats instead: {format}.",
    }
    datetime_parser = datetime.datetime.strptime

    def __init__(self, format=empty, input_formats=None, *args, **kwargs):
        if format is not empty:
            self.format = format
        if input_formats is not None:
            self.input_formats = input_formats
        super(TimeField, self).__init__(*args, **kwargs)

    def to_internal_value(self, value):
        input_formats = getattr(
            self, 'input_formats', settings.TIME_INPUT_FORMATS
        )

        if isinstance(value, datetime.time):
            return value

        for input_format in input_formats:
            if input_format.lower() == settings.ISO_8601:
                try:
                    parsed = dateparse.parse_time(value)
                except (ValueError, TypeError):
                    pass
                else:
                    if parsed is not None:
                        return parsed
            else:
                try:
                    parsed = self.datetime_parser(value, input_format)
                except (ValueError, TypeError):
                    pass
                else:
                    return parsed.time()

        humanized_format = humanize_datetime.time_formats(input_formats)
        self.raise_error('invalid', format=humanized_format)

    def to_representation(self, value):
        if not value:
            return None

        output_format = getattr(self, 'format', settings.TIME_FORMAT)

        if output_format is None:
            return value

        # Applying a `TimeField` to a datetime value is almost always not a
        # sensible thing to do, as it means naively dropping any explicit or
        # implicit timezone info
        assert not isinstance(value, datetime.datetime), (
            'Expected a `time`, but got a `datetime`. Refusing to coerce, '
            'as this may mean losing timezone information. Use a custom '
            'read-only field and deal with timezone issues explicitly.'
        )

        if output_format.lower() == settings.ISO_8601:
            if isinstance(value, str):
                value = datetime.datetime.strptime(value, '%H:%M:%S').time()
            return value.isoformat()
        return value.strftime(output_format)


class DecimalField(AbstractField):
    default_error_messages = {
        'invalid': u"A valid number is required.",
        'max_value': u"Ensure this value is less than or equal "
                     u"to {max_value}.",
        'min_value': u"Ensure this value is greater than or equal "
                     u"to {min_value}.",
        'max_digits': u"Ensure that there are no more than {max_digits} "
                      u"digits in total.",
        'max_decimal_places': u"Ensure that there are no more than "
                              u"{max_decimal_places} decimal places.",
        'max_whole_digits': u"Ensure that there are no more than "
                            u"{max_whole_digits} digits before the "
                            u"decimal point.",
        'max_string_length': u"String value too large."
    }
    MAX_STRING_LENGTH = 1000  # Guard against malicious string inputs

    def __init__(self, max_digits, decimal_places, coerce_to_string=None,
                 max_value=None, min_value=None, **kwargs):
        self.max_digits = max_digits
        self.decimal_places = decimal_places
        if coerce_to_string is not None:
            self.coerce_to_string = coerce_to_string

        self.max_value = max_value
        self.min_value = min_value

        if self.max_digits is not None and self.decimal_places is not None:
            self.max_whole_digits = self.max_digits - self.decimal_places
        else:
            self.max_whole_digits = None

        super(DecimalField, self).__init__(**kwargs)

        if self.max_value is not None:
            message = self.error_messages['max_value'].format(
                max_value=self.max_value
            )
            self.validators.append(
                validators.MaxValueValidator(self.max_value, message=message)
            )
        if self.min_value is not None:
            message = self.error_messages['min_value'].format(
                min_value=self.min_value
            )
            self.validators.append(
                validators.MinValueValidator(self.min_value, message=message)
            )

    def to_internal_value(self, data):
        """
        Validate that the input is a decimal number and return a Decimal
        instance.
        """
        data = force_text(data).strip()
        if len(data) > self.MAX_STRING_LENGTH:
            self.raise_error('max_string_length')

        try:
            value = decimal.Decimal(data)
        except decimal.DecimalException:
            self.raise_error('invalid')

        # Check for NaN. It is the only value that isn't equal to itself,
        # so we can use this to identify NaN values.
        if value != value:
            self.raise_error('invalid')

        # Check for infinity and negative infinity.
        if value in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):
            self.raise_error('invalid')

        return self.validate_precision(value)

    def validate_precision(self, value):
        """
        Ensure that there are no more than max_digits in the number, and no
        more than decimal_places digits after the decimal point.
        Override this method to disable the precision validation for input
        values or to enhance it in any way you need to.
        """
        sign, digittuple, exponent = value.as_tuple()

        if exponent >= 0:
            # 1234500.0
            total_digits = len(digittuple) + exponent
            whole_digits = total_digits
            decimal_places = 0
        elif len(digittuple) > abs(exponent):
            # 123.45
            total_digits = len(digittuple)
            whole_digits = total_digits - abs(exponent)
            decimal_places = abs(exponent)
        else:
            # 0.001234
            total_digits = abs(exponent)
            whole_digits = 0
            decimal_places = total_digits

        if self.max_digits is not None and total_digits > self.max_digits:
            self.raise_error('max_digits', max_digits=self.max_digits)
        if self.decimal_places is not None and \
                decimal_places > self.decimal_places:
            self.raise_error(
                'max_decimal_places', max_decimal_places=self.decimal_places
            )
        if self.max_whole_digits is not None and \
                whole_digits > self.max_whole_digits:
            self.raise_error(
                'max_whole_digits', max_whole_digits=self.max_whole_digits
            )

        return value

    def to_representation(self, value):
        coerce_to_string = getattr(
            self, 'coerce_to_string', settings.COERCE_DECIMAL_TO_STRING
        )

        if not isinstance(value, decimal.Decimal):
            value = decimal.Decimal(str(value).strip())

        quantized = self.quantize(value)

        if not coerce_to_string:
            return quantized
        return '{0:f}'.format(quantized)

    def quantize(self, value):
        """
        Quantize the decimal value to the configured precision.
        """
        context = decimal.getcontext().copy()
        context.prec = self.max_digits
        return value.quantize(
            decimal.Decimal('.1') ** self.decimal_places,
            context=context
        )


class DateField(AbstractField):
    default_error_messages = {
        'invalid': u"Date has wrong format. Use one of these "
                   u"formats instead: {format}.",
        'datetime': u"Expected a date but got a datetime.",
    }
    datetime_parser = datetime.datetime.strptime

    def __init__(self, format=empty, input_formats=None, *args, **kwargs):
        if format is not empty:
            self.format = format
        if input_formats is not None:
            self.input_formats = input_formats
        super(DateField, self).__init__(*args, **kwargs)

    def to_internal_value(self, value):
        input_formats = getattr(
            self, 'input_formats', settings.DATE_INPUT_FORMATS
        )

        if isinstance(value, datetime.datetime):
            self.raise_error('datetime')

        if isinstance(value, datetime.date):
            return value

        for input_format in input_formats:
            if input_format.lower() == settings.ISO_8601:
                try:
                    parsed = dateparse.parse_date(value)
                except (ValueError, TypeError):
                    pass
                else:
                    if parsed is not None:
                        return parsed
            else:
                try:
                    parsed = self.datetime_parser(value, input_format)
                except (ValueError, TypeError):
                    pass
                else:
                    return parsed.date()

        humanized_format = humanize_datetime.date_formats(input_formats)
        self.raise_error('invalid', format=humanized_format)

    def to_representation(self, value):
        if not value:
            return None

        output_format = getattr(self, 'format', settings.DATE_FORMAT)

        if output_format is None:
            return value

        # Applying a `DateField` to a datetime value is almost always not a
        # sensible thing to do, as it means naively dropping any explicit or
        # implicit timezone info
        assert not isinstance(value, datetime.datetime), (
            'Expected a `date`, but got a `datetime`. Refusing to coerce, '
            'as this may mean losing timezone information. Use a custom '
            'read-only field and deal with timezone issues explicitly.'
        )

        if output_format.lower() == settings.ISO_8601:
            if isinstance(value, str):
                value = datetime.datetime.strptime(value, '%Y-%m-%d').date()
            return value.isoformat()

        return value.strftime(output_format)


class DateTimeField(AbstractField):
    default_error_messages = {
        'invalid': u"Datetime has wrong format. Use one of these "
                   u"formats instead: {format}.",
        'date': u"Expected a datetime but got a date.",
    }
    datetime_parser = datetime.datetime.strptime

    def __init__(self, format=empty, input_formats=None, default_timezone=None,
                 *args, **kwargs):
        if format is not empty:
            self.format = format
        if input_formats is not None:
            self.input_formats = input_formats
        if default_timezone is not None:
            self.timezone = default_timezone
        super(DateTimeField, self).__init__(*args, **kwargs)

    def enforce_timezone(self, value):
        """
        When `self.timezone` is `None`, always return naive datetimes.
        When `self.timezone` is not `None`, always return aware datetimes.
        """
        field_timezone = getattr(self, 'timezone', self.default_timezone())

        if (field_timezone is not None) and not timezone.is_aware(value):
            return timezone.make_aware(value, field_timezone)
        elif (field_timezone is None) and timezone.is_aware(value):
            return timezone.make_naive(value, timezone.UTC())
        return value

    def default_timezone(self):
        return timezone.get_default_timezone() if settings.USE_TZ else None

    def to_internal_value(self, value):
        input_formats = getattr(
            self, 'input_formats', settings.DATETIME_INPUT_FORMATS
        )

        if isinstance(value, datetime.date) and \
                not isinstance(value, datetime.datetime):
            self.raise_error('date')

        if isinstance(value, datetime.datetime):
            return self.enforce_timezone(value)

        for input_format in input_formats:
            if input_format.lower() == settings.ISO_8601:
                try:
                    parsed = dateparse.parse_datetime(value)
                except (ValueError, TypeError):
                    pass
                else:
                    if parsed is not None:
                        return self.enforce_timezone(parsed)
            else:
                try:
                    parsed = self.datetime_parser(value, input_format)
                except (ValueError, TypeError):
                    pass
                else:
                    return self.enforce_timezone(parsed)

        humanized_format = humanize_datetime.datetime_formats(input_formats)
        self.raise_error('invalid', format=humanized_format)

    def to_representation(self, value):
        if not value:
            return None

        output_format = getattr(self, 'format', settings.DATETIME_FORMAT)

        if output_format is None:
            return value

        if output_format.lower() == settings.ISO_8601:
            value = value.isoformat()
            if value.endswith('+00:00'):
                value = value[:-6] + 'Z'
            return value
        return value.strftime(output_format)


class TimeDeltaField(AbstractField):
    ACCEPTABLE_DISPLAY_TYPES = ["sql", "iso8601", "minimal", "short", "long"]
    default_error_messages = {
        'timedelta': u"Expected a string of the form '1w, 2d, 5h, 45m, 5s'",
    }

    def __init__(self, **kwargs):
        self.display = kwargs.pop("display", "long")
        self.separator = kwargs.pop("separator", ", ")

        assert self.display in self.ACCEPTABLE_DISPLAY_TYPES, (
            "Attribute `display` can take one of the following values: "
            "'sql, 'iso8601', 'minimal', 'short' or 'long'."
        )

        super(TimeDeltaField, self).__init__(**kwargs)

    def to_internal_value(self, data):
        try:
            data = dateparse.parse_timedelta(data)
        except (AttributeError, TypeError):
            self.raise_error('timedelta', type=type(data))
        return data

    def to_representation(self, value):
        return humanize_datetime.humanize_timedelta(
            value, display=self.display, sep=self.separator
        )


class HiddenField(AbstractField):
    """
    A hidden field does not take input from the user, or present any output,
    but it does populate a field in `validated_data`, based on its default
    value. This is particularly useful when we have a `unique_for_date`
    constraint on a pair of fields, as we need some way to include the date in
    the validated data.
    """
    def __init__(self, **kwargs):
        assert 'default' in kwargs, 'default is a required argument.'
        kwargs['write_only'] = True
        super(HiddenField, self).__init__(**kwargs)

    def get_value(self, dictionary):
        # We always use the default value for `HiddenField` and user input
        # is never provided or accepted
        return empty

    def to_internal_value(self, data):
        return data


class CreateOnlyDefault(object):
    """
    This class may be used to provide default values that are only used
    for create operations, but that do not return any value for update
    operations.
    """
    def __init__(self, default):
        self.default = default

    def _can_set_context(self):
        return hasattr(self.default, 'set_context') and not self.is_update

    def set_context(self, serializer_field):
        self.is_update = serializer_field.parent.instance is not None
        if callable(self.default) and self._can_set_context():
            self.default.set_context(serializer_field)

    def __call__(self):
        if self.is_update:
            raise SkipField()
        elif callable(self.default):
            return self.default()
        return self.default

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self.default)


class ReadOnlyField(AbstractField):
    """
    A read-only field that simply returns the field value.
    If the field is a method with no parameters, the method will be called
    and it's return value used as the representation.
    For example, the following would call `get_expiry_date()` on the object:

    class ExampleSerializer(Serializer):
        expiry_date = ReadOnlyField(source='get_expiry_date')
    """
    def __init__(self, **kwargs):
        kwargs['read_only'] = True
        super(ReadOnlyField, self).__init__(**kwargs)

    def to_representation(self, value):
        return value


class _UnvalidatedField(AbstractField):
    def __init__(self, *args, **kwargs):
        super(_UnvalidatedField, self).__init__(*args, **kwargs)
        self.allow_blank = True
        self.allow_null = True

    def to_internal_value(self, data):
        return data

    def to_representation(self, value):
        return value


class ListField(AbstractField):
    child = _UnvalidatedField()
    initial = []
    default_error_messages = {
        'not_a_list': u'Expected a list of items but got type "{input_type}".',
        'empty': u"This list may not be empty."
    }

    def __init__(self, *args, **kwargs):
        self.child = kwargs.pop('child', copy.deepcopy(self.child))
        self.allow_empty = kwargs.pop('allow_empty', True)

        assert not inspect.isclass(self.child), (
            '`child` has not been instantiated.'
        )
        assert self.child.source is None, (
            "The `source` argument is not meaningful when applied to a "
            "`child=` field. Remove `source=` from the field declaration."
        )

        super(ListField, self).__init__(*args, **kwargs)
        self.child.bind(field_name='', parent=self)

    def get_value(self, dictionary):
        if self.field_name not in dictionary:
            if getattr(self.root, 'partial', False):
                return empty
        return dictionary.get(self.field_name, empty)

    def to_internal_value(self, data):
        """
        List of dicts of native values <- List of dicts of primitive datatypes.
        """
        if isinstance(data, str) or isinstance(data, collections.Mapping) \
                or not hasattr(data, '__iter__'):
            self.raise_error('not_a_list', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            self.raise_error('empty')
        return [self.child.run_validation(item) for item in data]

    def to_representation(self, data):
        """
        List of object instances -> List of dicts of primitive datatypes.
        """
        return [
            self.child.to_representation(item) if item is not None else None
            for item in data
        ]


class DictField(AbstractField):
    child = _UnvalidatedField()
    initial = {}
    default_error_messages = {
        'not_a_dict': u'Expected a dictionary of items but got '
                      u'type "{input_type}".'
    }

    def __init__(self, *args, **kwargs):
        self.child = kwargs.pop('child', copy.deepcopy(self.child))

        assert not inspect.isclass(self.child), (
            '`child` has not been instantiated.'
        )
        assert self.child.source is None, (
            "The `source` argument is not meaningful when applied to "
            "a `child=` field. Remove `source=` from the field declaration."
        )

        super(DictField, self).__init__(*args, **kwargs)
        self.child.bind(field_name='', parent=self)

    def get_value(self, dictionary):
        return dictionary.get(self.field_name, empty)

    def to_internal_value(self, data):
        """
        Dicts of native values <- Dicts of primitive datatypes.
        """
        if not isinstance(data, dict):
            self.raise_error('not_a_dict', input_type=type(data).__name__)
        return {
            str(key): self.child.run_validation(value)
            for key, value in data.items()
        }

    def to_representation(self, value):
        """
        List of object instances -> List of dicts of primitive datatypes.
        """
        return {
            str(key): self.child.to_representation(val) if val is not None else None  # NOQA
            for key, val in value.items()
        }


class HStoreField(DictField):
    """
    HStore field for PostgreSQL.
    """
    child = CharField(allow_blank=True)


class JSONField(AbstractField):
    """
    Special kind of field which is storing data in JSON format.
    """
    default_error_messages = {
        'invalid': u"Value must be valid JSON."
    }

    def __init__(self, *args, **kwargs):
        self.binary = kwargs.pop('binary', False)
        super(JSONField, self).__init__(*args, **kwargs)

    def to_internal_value(self, data):
        try:
            if self.binary:
                if isinstance(data, bytes):
                    data = data.decode('utf-8')
                return json.loads(data)
            else:
                json.dumps(data)
        except (TypeError, ValueError):
            self.raise_error('invalid')
        return data

    def to_representation(self, value):
        if self.binary:
            value = json.dumps(value)
            if isinstance(value, str):
                value = bytes(value.encode('utf-8'))
        return value


class ModelField(AbstractField):
    """
    A generic field that can be used against an arbitrary model field.
    This is used by `ModelSerializer` when dealing with custom model fields,
    that do not have a serializer field to be mapped to.
    """
    def __init__(self, model_field, **kwargs):
        self.model_field = model_field
        super(ModelField, self).__init__(**kwargs)

    def to_internal_value(self, data):
        raise NotImplementedError('`to_internal_value()` must be implemented.')

    def get_attribute(self, obj):
        # We pass the object instance onto `to_representation`,
        # not just the field attribute
        return obj

    def to_representation(self, obj):
        raise NotImplementedError('`to_representation()` must be implemented.')


class SerializerMethodField(AbstractField):
    """
    A read-only field that get its representation from calling a method on the
    parent serializer class. The method called will be of the form
    "get_{field_name}", and should take a single argument, which is the
    object being serialized. For example:

    class ExampleSerializer(ModelSerializer):
        extra_info = SerializerMethodField()
        def get_extra_info(self, obj):
            return ...  # Calculate some data to return
    """
    def __init__(self, method_name=None, **kwargs):
        self.method_name = method_name
        kwargs['source'] = '*'
        kwargs['read_only'] = True
        super(SerializerMethodField, self).__init__(**kwargs)

    def bind(self, field_name, parent):
        # In order to enforce a consistent style, we error if a redundant
        # 'method_name' argument has been used. For example:
        # my_field = serializer.CharField(source='my_field')
        default_method_name = 'get_{field_name}'.format(field_name=field_name)
        assert self.method_name != default_method_name, (
            "It is redundant to specify `%s` on SerializerMethodField '%s' in "
            "serializer '%s', because it is the same as the default method "
            "name. Remove the `method_name` argument." %
            (self.method_name, field_name, parent.__class__.__name__)
        )

        # The method name should default to `get_{field_name}`
        if self.method_name is None:
            self.method_name = default_method_name

        super(SerializerMethodField, self).bind(field_name, parent)

    def to_representation(self, value):
        method = getattr(self.parent, self.method_name)
        return method(value)



# -*- coding: utf-8 -*-
"""
Special validator classes and functions, applied for checking passed data.
"""
from aiorest_ws.db.orm.exceptions import ValidationError
from aiorest_ws.db.orm.sqlalchemy.mixins import ORMSessionMixin
from aiorest_ws.db.orm.validators import BaseValidator, \
    BaseUniqueFieldValidator

__all__ = ('ORMFieldValidator', 'UniqueORMValidator', )


class ORMFieldValidator(BaseValidator):
    message = u"Assertion error for `{field}` field with `{value}` value."

    def __init__(self, validator_func, model_field, field_name, **kwargs):
        super(ORMFieldValidator, self).__init__(**kwargs)
        self.validator_func = validator_func
        self.model_field = model_field
        self.field_name = field_name

    def __call__(self, value):
        try:
            self.validator_func(self.model_field, self.field_name, value)
        except AssertionError as exc:
            message = exc.args[0] if exc.args else self.message.format(
                field=self.field_name, value=value
            )
            raise ValidationError(message)


class UniqueORMValidator(BaseUniqueFieldValidator, ORMSessionMixin):
    """
    Validator that corresponds to `unique=True` on a model field.
    Should be applied to an individual field on the serializer.
    """
    def __init__(self, model, field_name, message=None):
        super(UniqueORMValidator, self).__init__(model, message)
        self.model = model
        self.field_name = field_name
        self.instance = None

    def filter_queryset(self, value, queryset):
        filter_field = getattr(self.model, self.field_name)
        return queryset.filter(filter_field == value)

    def exclude_current_instance(self, queryset):
        """
        If an instance is being updated, then do not include
        that instance itself as a uniqueness conflict.
        """
        if self.instance:
            pk_fields = [
                column.name
                for column in self.instance.__mapper__.primary_key
            ]
            filter_args = [
                getattr(self.model, attr) != getattr(self.instance, attr)
                for attr in pk_fields
            ]
            return queryset.filter(*filter_args)
        return queryset

    def __call__(self, value):
        session = self._get_session()
        try:
            queryset = session.query(self.model)
            queryset = self.filter_queryset(value, queryset)
            queryset = self.exclude_current_instance(queryset)

            if session.query(queryset.exists()).scalar():
                raise ValidationError(self.message)
        finally:
            session.close()



# -*- coding: utf-8 -*-
"""
Wrappers, similar on HTTP requests/responses.
"""
from aiorest_ws.utils.modify import add_property

__all__ = ('Request', 'Response', )


class Request(object):

    def __init__(self, *args, **kwargs):
        super(Request, self).__init__()
        self._method = kwargs.pop('method', None)
        self._url = kwargs.pop('url', None)
        self._args = kwargs.pop('args', {})
        self._data = kwargs.pop('data', None)
        self._event_name = kwargs.pop('event_name', None)

        for key in kwargs.keys():
            add_property(self, key, kwargs[key])

    @property
    def method(self):
        """
        Get method type, which defined by the user.
        """
        return self._method

    @property
    def url(self):
        """
        Get the APIs url, from which expected response.
        """
        return self._url

    @property
    def args(self):
        """
        Get dictionary of arguments, defined by the user.
        """
        return self._args

    @property
    def data(self):
        """
        Get request body.
        """
        return self._data

    @property
    def event_name(self):
        """
        Get event name, which used by the client for processing response.
        """
        return self._event_name

    def to_representation(self):
        """
        Serialize request object to dictionary object.
        """
        return {'event_name': self.event_name}

    def get_argument(self, name):
        """
        Extracting argument from the request.

        :param name: name of extracted argument in dictionary.
        """
        return self.args.get(name, None) if self.args else None


class Response(object):

    def __init__(self):
        super(Response, self).__init__()
        self._content = {}

    @property
    def content(self):
        """
        Get content of response.
        """
        return self._content

    @content.setter
    def content(self, value):
        """
        Set content for the response.
        """
        self._content['data'] = value

    def wrap_exception(self, exception):
        """
        Set content of response, when taken exception.
        """
        self._content = {'detail': exception.detail}

    def append_request(self, request):
        """
        Add to the response object serialized request.
        """
        self._content.update(request.to_representation())



# -*- coding: utf-8 -*-
"""
Endpoint classes for aiorest-ws router.
"""
from aiorest_ws.abstract import AbstractEndpoint

__all__ = ('PlainEndpoint', 'DynamicEndpoint', )


class PlainEndpoint(AbstractEndpoint):

    def match(self, path):
        """
        Checking path on compatible.

        :param path: URL, which used for get access to API.
        """
        match_result = None
        if self.path == path:
            match_result = ()
        return match_result


class DynamicEndpoint(AbstractEndpoint):

    def __init__(self, path, methods, handler, name, pattern):
        super(DynamicEndpoint, self).__init__(path, methods, handler, name)
        self._pattern = pattern

    def match(self, path):
        """
        Checking path on compatible.

        :param path: URL, which used for get access to API.
        """
        match_result = self._pattern.match(path)
        # If comparing has successful, then return list of parsed values
        if match_result:
            match_result = match_result.groups()
        return match_result



# -*- coding: utf-8 -*-
"""
Special functions for additional and safety encoding passed data.
"""
import datetime
from decimal import Decimal

from aiorest_ws.utils.serializer_helpers import ReturnList, ReturnDict

__all__ = (
    '_PROTECTED_TYPES', 'is_protected_type', 'force_text',
    'force_text_recursive',
)

_PROTECTED_TYPES = (
    str, type(None), float, Decimal, datetime.datetime, datetime.date,
    datetime.time
)


def is_protected_type(obj):
    """
    Determine if the object instance is of a protected type.
    Objects of protected types are preserved as-is when passed to
    force_text(strings_only=True).
    """
    return isinstance(obj, _PROTECTED_TYPES)


def force_text(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Similar to smart_text, except that lazy instances are resolved to
    strings, rather than kept as lazy objects.
    If strings_only is True, don't convert (some) non-string-like objects.
    """
    # Handle the common case first for performance reasons
    if issubclass(type(s), str):
        return s
    if strings_only and is_protected_type(s):
        return s
    try:
        if not issubclass(type(s), str):
            if isinstance(s, bytes):
                s = str(s, encoding, errors)
            else:
                s = str(s)
        else:
            # Note: We use .decode() here, instead of six.text_type(s,
            # encoding, errors), so that if s is a SafeBytes, it ends up
            # being a SafeText at the end
            s = s.decode(encoding, errors)
    except UnicodeDecodeError:
        # If we get to here, the caller has passed in an Exception
        # subclass populated with non-ASCII bytestring data without a
        # working unicode method. Try to handle this without raising a
        # further exception by individually forcing the exception args
        # to unicode
        s = ' '.join(force_text(arg, encoding, strings_only, errors)
                     for arg in s)
    return s


def force_text_recursive(data):
    """
    Descend into a nested data structure, forcing any
    lazy translation strings into plain text.
    """
    if isinstance(data, list):
        ret = [force_text_recursive(item) for item in data]
        if isinstance(data, ReturnList):
            return ReturnList(ret, serializer=data.serializer)
        return data
    elif isinstance(data, dict):
        ret = {
            key: force_text_recursive(value)
            for key, value in data.items()
        }
        if isinstance(data, ReturnDict):
            return ReturnDict(ret, serializer=data.serializer)
        return data
    return force_text(data)



# -*- coding: utf-8 -*-
"""
Validator classes and functions for checking passed arguments.
"""
import inspect

from aiorest_ws.exceptions import NotSupportedArgumentType, \
    InvalidPathArgument, InvalidHandler
from aiorest_ws.views import MethodBasedView
from aiorest_ws.utils.validators import to_str, get_object_type

__all__ = (
    'BaseValidator', 'EndpointNameValidator', 'HandlerValidator',
    'MethodValidator', 'PathValidator', 'RouteArgumentsValidator',
    'check_and_set_subclass',
)


class BaseValidator(object):
    """
    Base class for validators.
    """

    def validate(self, *args, **kwargs):
        """
        Validating passed arguments and kwargs.
        """
        pass


class EndpointNameValidator(BaseValidator):
    """
    Validator for endpoint name argument.
    """
    def _check_name(self, name):
        """
        Validate passed name variable.

        :param name: the base to use for the URL names that are created.
        """
        if name and type(name) is not str:
            raise NotSupportedArgumentType(u'name variable must '
                                           u'be string type')

    def validate(self, name):
        """
        Validate passed name variable.

        :param name: the base to use for the URL names that are created.
        """
        self._check_name(name)


class HandlerValidator(BaseValidator):
    """
    Validator for handler argument.
    """
    def _check_handler(self, handler):
        """
        Validate passed handler for requests.

        :param handler: class or function (wrapped into MethodBasedView),
                        which used for generating response.
        """
        if inspect.isclass(handler):
            if not issubclass(handler, (MethodBasedView, )):
                raise InvalidHandler(u"Your class should be inherited from "
                                     u"the MethodBasedView class")
        else:
            raise InvalidHandler()

    def validate(self, handler):
        """
        Validate passed handler for requests.

        :param handler: class or function, used for generating response.
        """
        self._check_handler(handler)


class MethodValidator(BaseValidator):
    """
    Validator for methods argument.
    """
    supported_methods_types = (list, str)

    def _check_methods(self, methods):
        """
        Validate passed methods variable.

        :param methods: list of methods or string with concrete method name.
        """
        if type(methods) not in self.supported_methods_types:
            raise NotSupportedArgumentType('Variable with name "methods" must'
                                           ' be `list` or `str` type.')

    def validate(self, methods):
        """Validate passed methods variable.

        :param methods: list of methods or string with concrete method name.
        """
        self._check_methods(methods)


class PathValidator(BaseValidator):
    """
    Validator for path argument.
    """
    def _check_path(self, path):
        """
        Validate passed path for endpoint.

        :param path: path to endpoint (string).
        """
        if not path.startswith('/'):
            raise InvalidPathArgument(u"Path should be started with `/` "
                                      u"symbol")

    def validate(self, path):
        """
        Validate passed path for endpoint.

        :param path: path to endpoint (string).
        """
        self._check_path(path)


class RouteArgumentsValidator(BaseValidator):
    """
    Validator for arguments of RestWSRouter.
    """
    path_validator = PathValidator()
    handler_validator = HandlerValidator()
    methods_validator = MethodValidator()
    endpoint_name_validator = EndpointNameValidator()

    def validate(self, path, handler, methods, name):
        """
        Validating passed arguments and kwargs.
        """
        self.path_validator.validate(path)
        self.handler_validator.validate(handler)
        self.methods_validator.validate(methods)
        self.endpoint_name_validator.validate(name)


def check_and_set_subclass(instance, attribute, value, subclasses):
    """
    Validate subclass of passed value on supported type and set him for
    instance of some class.

    :param instance: object, for with necessary make check and sets value.
    :param attribute: name of attribute (string), at which write ``value``
                      when checking passed without any errors.
    :param value: passed value to validate.
    :param subclasses: class or a list/tuple with acceptable classes.
    """
    obj_type = get_object_type(value)
    if issubclass(obj_type, subclasses):
        setattr(instance, attribute, value)
    else:
        raise TypeError('Custom class must be inherited from the '
                        '{} class.'.format(to_str(subclasses)))



# -*- coding: utf-8 -*-
"""
Helper function for returning the field information that is associated
with a model class. This includes returning all the forward and reverse
relationships and their associated metadata.
"""
from collections import OrderedDict

from aiorest_ws.utils.structures import FieldInfo, RelationInfo
from aiorest_ws.db.orm.django.compat import get_related_model, \
    get_remote_field

__all__ = (
    '_get_pk', '_get_fields', '_get_to_field', '_get_forward_relationships',
    '_get_reverse_relationships', '_merge_fields_and_pk',
    '_merge_relationships', 'get_field_info', 'is_abstract_model'
)


def _get_pk(opts):
    pk = opts.pk
    rel = get_remote_field(pk)

    while rel and rel.parent_link:
        # If model is a child via multi-table inheritance, use parent's pk
        pk = get_related_model(pk)._meta.pk
        rel = get_remote_field(pk)

    return pk


def _get_fields(opts):
    fields = OrderedDict()
    opts_fields = [
        field for field in opts.fields
        if field.serialize and not get_remote_field(field)
    ]
    for field in opts_fields:
        fields[field.name] = field

    return fields


def _get_to_field(field):
    return getattr(field, 'to_fields', None) and field.to_fields[0]


def _get_forward_relationships(opts):
    """
    Returns an `OrderedDict` of field names to `RelationInfo`.
    """
    forward_relations = OrderedDict()
    forwards_fields = [
        field for field in opts.fields
        if field.serialize and get_remote_field(field)
    ]
    for field in forwards_fields:
        forward_relations[field.name] = RelationInfo(
            model_field=field,
            related_model=get_related_model(field),
            to_many=False,
            to_field=_get_to_field(field),
            has_through_model=False
        )

    # Deal with forward many-to-many relationships.
    many_to_many_fields = [
        field for field in opts.many_to_many
        if field.serialize
    ]
    for field in many_to_many_fields:
        forward_relations[field.name] = RelationInfo(
            model_field=field,
            related_model=get_related_model(field),
            to_many=True,
            # many-to-many do not have to_fields
            to_field=None,
            has_through_model=not get_remote_field(field).through._meta.auto_created  # NOQA
        )

    return forward_relations


def _get_reverse_relationships(opts):
    """
    Returns an `OrderedDict` of field names to `RelationInfo`.
    """
    reverse_relations = OrderedDict()
    all_related_objects = [
        r for r in opts.related_objects
        if not r.field.many_to_many
    ]
    for relation in all_related_objects:
        accessor_name = relation.get_accessor_name()
        related = getattr(relation, 'related_model', relation.model)
        reverse_relations[accessor_name] = RelationInfo(
            model_field=None,
            related_model=related,
            to_many=get_remote_field(relation.field).multiple,
            to_field=_get_to_field(relation.field),
            has_through_model=False
        )

    # Deal with reverse many-to-many relationships.
    all_related_many_to_many_objects = [
        r for r in opts.related_objects
        if r.field.many_to_many
    ]
    for relation in all_related_many_to_many_objects:
        has_through_model = False
        through = getattr(get_remote_field(relation.field), 'through', None)
        if through is not None:
            remote_field = get_remote_field(relation.field)
            has_through_model = not remote_field.through._meta.auto_created

        accessor_name = relation.get_accessor_name()
        related = getattr(relation, 'related_model', relation.model)
        reverse_relations[accessor_name] = RelationInfo(
            model_field=None,
            related_model=related,
            to_many=True,
            # many-to-many do not have to_fields
            to_field=None,
            has_through_model=has_through_model
        )

    return reverse_relations


def _merge_fields_and_pk(pk, fields):
    fields_and_pk = OrderedDict()
    fields_and_pk['pk'] = pk
    fields_and_pk[pk.name] = pk
    fields_and_pk.update(fields)

    return fields_and_pk


def _merge_relationships(forward_relations, reverse_relations):
    return OrderedDict(
        list(forward_relations.items()) +
        list(reverse_relations.items())
    )


def get_field_info(model):
    """
    Given a model class, returns a `FieldInfo` instance, which is a
    `namedtuple`, containing metadata about the various field types on the
    model including information about their relationships.
    """
    opts = model._meta.concrete_model._meta

    pk = _get_pk(opts)
    fields = _get_fields(opts)
    forward_relations = _get_forward_relationships(opts)
    reverse_relations = _get_reverse_relationships(opts)
    fields_and_pk = _merge_fields_and_pk(pk, fields)
    relationships = _merge_relationships(forward_relations, reverse_relations)

    return FieldInfo(pk, fields, forward_relations, reverse_relations,
                     fields_and_pk, relationships)


def is_abstract_model(model):
    """
    Given a model class, returns a boolean True if it is abstract and False
    if it is not.
    """
    has_meta_attribute = hasattr(model, '_meta')
    is_abstract = hasattr(model._meta, 'abstract') and model._meta.abstract
    return has_meta_attribute and is_abstract



# -*- coding: utf-8 -*-
"""
Storage backends for save/get credentials.
"""
from aiorest_ws.auth.token.utils import SQL_CREATE_TOKEN_TABLE, \
    SQL_TOKEN_GET, SQL_TOKEN_GET_BY_TOKEN_USERNAME, SQL_TOKEN_ADD, \
    TOKEN_MODEL_FIELDS
from aiorest_ws.conf import settings
from aiorest_ws.db.backends.sqlite3.constants import IN_MEMORY
from aiorest_ws.db.backends.sqlite3.managers import SQLiteManager
from aiorest_ws.db.utils import convert_db_row_to_dict
from aiorest_ws.storages.backends import BaseStorageBackend

__all__ = ('InMemoryTokenBackend', )


class InMemoryTokenBackend(BaseStorageBackend):
    """
    In memory backend (based on SQLite) for token.
    """

    def __init__(self):
        super(InMemoryTokenBackend, self).__init__()
        if settings.DATABASES:
            self.db_manager = settings.DATABASES['default']['manager']
        else:
            self.db_manager = SQLiteManager(name=IN_MEMORY)
        self.__create_models()

    def __create_models(self):
        self.db_manager.execute_script(SQL_CREATE_TOKEN_TABLE)

    def get(self, token):
        """
        Get token from the storage.

        :param token: token as string.
        """
        args = (token, )
        row = self.db_manager.execute_sql_and_fetchone(SQL_TOKEN_GET, args)
        if row:
            token_object = convert_db_row_to_dict(row, TOKEN_MODEL_FIELDS)
        else:
            token_object = {}
        return token_object

    def get_token_by_username(self, token_name, username):
        """
        Get token from the storage by token_name and username.

        :param token: token as string.
        """
        args = (token_name, username)
        row = self.db_manager.execute_sql_and_fetchone(
            SQL_TOKEN_GET_BY_TOKEN_USERNAME, args
        )
        if row:
            token_object = convert_db_row_to_dict(row, TOKEN_MODEL_FIELDS)
        else:
            token_object = {}
        return token_object

    def save(self, token_name, token, expired=None, user_id=None):
        """
        Save token in the storage.

        :param user: instance of User class.
        :param token: token as string.
        """
        args = (token_name, token, expired, user_id)
        self.db_manager.execute_sql(SQL_TOKEN_ADD, args)



# -*- coding: utf-8 -*-
"""
Manager classes and functions, which help for work with databases via SQL.
"""
__all__ = ('SQLiteManager', )

import sqlite3


class SQLiteManager(object):

    def __init__(self, *args, **kwargs):
        super(SQLiteManager, self).__init__()
        db_path = kwargs.get('name')
        self.connection = sqlite3.connect(db_path)

    def execute_sql(self, sql, parameters=()):
        """
        Executes a SQL statement.

        :param sql: SQL statement as string.
        :param parameters: tuple with arguments.
        """
        return self.connection.execute(sql, parameters)

    def execute_sql_and_fetchone(self, sql, parameters=()):
        """
        Executes a SQL statement with fetching one row from result.

        :param sql: SQL statement as string.
        :param parameters: tuple with arguments.
        """
        return self.execute_sql(sql, parameters).fetchone()

    def execute_sql_from_file(self, filepath):
        """
        Executes a SQL statement, which was taken from the file.

        :param filepath: path to file.
        """
        with open(filepath, 'r') as f:
            sql = f.read()
            result = self.connection.executescript(sql)
        return result

    def execute_script(self, sql):
        """
        Execute a SQL statement. This method recommended to use, when
        required to execute few SQL queries independent and in parallel.

        :param sql: SQL statement as string.
        """
        return self.connection.executescript(sql)
